!= ϳز (ڥȥˡ, Arakawa and Suarez (1983))
!
!= Dynamical Core (Spectral method, Arakawa and Suarez (1983))
!
! Authors::   Yasuhiro MORIKAWA
! Version::   $Id: dyn_spectral_as83.f90,v 1.29 2008-06-03 11:16:55 morikawa Exp $
! Tag Name::  $Name: dcpam4-20080609-1 $
! Copyright:: Copyright (C) GFD Dennou Club, 2007. All rights reserved.
! License::   See COPYRIGHT[link:../../../COPYRIGHT]
!

module dyn_spectral_as83
  !
  != ϳز (ڥȥˡ, Arakawa and Suarez (1983))
  !
  != Dynamical Core (Spectral method, Arakawa and Suarez (1983))
  !
  ! <b>Note that Japanese and English are described in parallel.</b>
  !
  ! ϳز黻⥸塼Ǥ. 
  ! ʿΥ˥ڥȥˡ, ľΥˤ Arakawa and Suarez (1983)
  ! ѤƤޤ.
  !
  ! ʬˡˤϥ꡼ץեåѤƤޤ.
  ! ǥեȤǤ, $ \Delta t $ 礭Ȥ뤿, ȹ 
  ! ߥץꥷåˡŬѤƤޤ.
  ! ȹ򥨥ץꥷåˡˤäƲ򤯤ȤǽǤ.
  ! ܤ Create  NAMELIST#dyn_as83_nml 򻲾ȤƤ.
  !
  ! This is a dynamical core module. Spectral method (for horizontal) and 
  ! Arakawa and Suarez (1983) method (for vertical) are used.
  !
  ! Leap-frog scheme is used as time integration. 
  ! By default, semi-implicit scheme is applied to gravitational terms 
  ! for extension of $ \Delta t $ . 
  ! Explicit scheme can be applied to gravitational terms. 
  ! For details, see "Create" or "NAMELIST#dyn_as83_nml". 
  !
  !== Procedures List
  !
  ! DynSpAsCreate        :: DYNSPAS83 ѿν
  ! Dynamics             :: ϳزα黻
  ! VorDiv2UV            :: ȯ®Ѵ
  ! UV2VorDiv            :: ®ȯѴ
  ! DynSpAsEqualAxes     :: DYNSPAS83 ѿ˳ǼƤɸͤγǧ
  ! DynSpAsGetAxes       :: ɸͤμ
  ! DynSpAsClose         :: DYNSPAS83 ѿνλ
  ! DynSpAsPutLine       :: DYNSPAS83 ѿ˳ǼƤΰ
  ! DynSpAsInitialized   :: DYNSPAS83 ѿꤵƤ뤫ݤ
  ! DynSpAsSetTime       :: 
  ! ------------  :: ------------
  ! DynSpAsCreate        :: Constructor of "DYNSPAS83"
  ! Dynamics             :: Calculation of dynamical core
  ! VorDiv2UV            :: Convert vorticity and divergence into wind
  ! UV2VorDiv            :: Convert wind into vorticity and divergence
  ! DynSpAsEqualAxes     :: Confirm data of axes in "DYNSPAS83"
  ! DynSpAsGetAxes       :: Get data of axes
  ! DynSpAsClose         :: Deconstructor of "DYNSPAS83"
  ! DynSpAsPutLine       :: Print information of "DYNSPAS83"
  ! DynSpAsInitialized   :: Check initialization of "DYNSPAS83"
  ! DynSpAsSetTime       :: Configure time
  !
  !== Usage
  !
  ! ץλϤ, DYNSPAS83 ѿ Create ǽޤ.
  ! , ʬ롼Ǳ, ȯ, , 漾, ɽ̵ 
  ! ( $ t $  $ t-\Delta t $ )  Dynamics ͿƤ. 
  ! ͤѤϳزα黻Ԥ, 
  ! $ t+\Delta t $ ®, ®, , ȯ, , 漾, 
  ! ɽ̵ޤ.
  ! Ǹ, DYNSPAS83 ѿνλ Close ˤƹԤޤ.
  ! ץǺɸͤɬפȤʤˤ, GetAxes Ѥ
  ! ɸͤƤ.
  !
  ! In the begging of program, initialize "DYNSPAS83" by "Create".
  ! Next, in the time integration loop, 
  ! provide vorticity, divergence, temperature, specific humidity,
  ! surface pressure on $ t $ and $ t-\Delta t $ to Dynamics.
  ! Then the physical values at $ t+\Delta t $ is returned.
  ! Finally, terminate "DYNSPAS83" by "Close".
  ! If you need data of axes, get that by GetAxes.
  !
  !== References
  !
  ! * Arakawa, A., Suarez, M. J., 1983:
  !   Vertical differencing of the primitive equations
  !   in sigma coordinates.
  !   <i>Mon. Wea. Rev.</i>, <b>111</b>, 34--35.

  use dc_types, only: DP, TOKEN, STRING
  use dyn_spectral, only: DYNSP
  use dyn_as83, only: DYNAS83
  use dc_date_types, only: DC_DIFFTIME
  use gt4_history_nmlinfo, only: GTHST_NMLINFO
  use gt4_history, only: GT_HISTORY
  implicit none
  private
  public:: DYNSPAS83
  public:: DynSpAsCreate, DynSpAsClose, DynSpAsPutLine, DynSpAsInitialized
  public:: Dynamics, VorDiv2UV, UV2VorDiv
  public:: DynSpAsEqualAxes, DynSpAsGetAxes

  type DYNSPAS83
    !
    ! ޤ, Create  "DYNSPAS83" ѿꤷƲ.
    ! 줿 "DYNSPAS83" ѿѤݤˤ,
    ! Close ˤäƽλԤäƤ.
    !
    ! Initialize "DYNSPAS83" variable by "Create" before usage.
    ! If you reuse "DYNSPAS83" variable again for another application, 
    ! terminate by "Close".
    !
    logical:: initialized = .false.     ! ե饰. 
                                        ! Initialization flag

    !-----------------------------------------------------------------
    !  ʻȿ
    !  Grid points and maximum truncated wavenumber
    !-----------------------------------------------------------------
    integer:: nmax ! ȿ. 
                   ! Maximum truncated wavenumber
    integer:: imax ! ٳʻ. 
                   ! Number of grid points in longitude
    integer:: jmax ! ٳʻ. 
                   ! Number of grid points in latitude
    integer:: kmax ! ľؿ. 
                   ! Number of vertical level

    type(DYNSP):: dyn_sp
    type(DYNAS83):: dyn_as

    !-----------------------------------------------------------------
    !  
    !  Time control
    !-----------------------------------------------------------------
    type(DC_DIFFTIME):: current_time
                              ! ߻. Current time.
    type(DC_DIFFTIME):: delta_time
                              ! $ Delta t $ . ॹƥå. Time step

    !-----------------------------------------------------------------
    !  ҥȥեؤΥǡ
    !  Configure the settings for history data output
    !-----------------------------------------------------------------
    type(GTHST_NMLINFO):: gthstnml
                              ! NAMELIST#dcmodel_sample_code_history_nml 
                              ! ꤵ̤ΥǡϾ. 
                              ! 
                              ! Individual data output information from 
                              ! "NAMELIST#dcmodel_sample_code_history_nml". 
  end type DYNSPAS83

  character(*), parameter:: version = &
    & '$Name: dcpam4-20080609-1 $' // &
    & '$Id: dyn_spectral_as83.f90,v 1.29 2008-06-03 11:16:55 morikawa Exp $'

  !-----------------------------------------------------------------
  !  ³
  !  Public procedures
  !-----------------------------------------------------------------

  interface DynSpAsCreate
    module procedure DynSpAsCreate
  end interface

  interface Dynamics
    module procedure DynSpAsDynamics
  end interface

  interface VorDiv2UV
    module procedure DynSpAsVorDiv2UV
  end interface

  interface UV2VorDiv
    module procedure DynSpAsUV2VorDiv
  end interface

  interface DynSpAsEqualAxes
    module procedure DynSpAsEqualAxes
  end interface

  interface DynSpAsGetAxes
    module procedure DynSpAsGetAxes
  end interface

  interface DynSpAsClose
    module procedure DynSpAsClose
  end interface

  interface DynSpAsPutLine
    module procedure DynSpAsPutLine
  end interface

  interface DynSpAsInitialized
    module procedure DynSpAsInitialized
  end interface

  interface DynSpAsSetTime
    module procedure DynSpAsSetTime
  end interface

  !-----------------------------------------------------------------
  !  ³
  !  Private procedures
  !-----------------------------------------------------------------

  interface NmlRead
    module procedure DynSpAsNmlRead
  end interface

contains

  subroutine DynSpAsCreate( dyn_sp_as, &
    & nmax, imax, jmax, kmax, &
    & PI, RPlanet, Omega, Cp, RAir, EpsV, VisOrder, EFoldTime, &
!!$    & Grav, &
    & DelTime, &
    & xy_Phis, &
    & openmp_threads, &
    & r_SigmaSet, &
    & wa_module_initialized, &
    & time_integration_scheme, &
    & current_time_value, current_time_unit, &
    & history_varlist, &
    & history_interval_value, history_interval_unit, &
    & history_precision, history_fileprefix, &
    & nmlfile, err)
    !
    !  *dyn_sp_as* ϳزԤ.
    ! ¾Υ֥롼ѤɬΥ֥롼ˤä
    ! DYNSPAS83 ѿꤷƤ.
    !
    ! NAMELIST Ѥˤϰ *nmlfile*  NAMELIST ե̾
    ! ͿƤ. NAMELIST ѿξܺ٤˴ؤƤ 
    ! NAMELIST#dyn_spectral_as83_nml 򻲾ȤƤ. 
    !
    ! Configure the settings for dynamical core to *dyn_sp_as*.
    ! Before other subroutines are used, initialize "DYNSPAS83"
    ! variable.
    !
    ! In order to use NAMELIST, specify a NAMELIST filename to 
    ! argument *nmlfile*. See "NAMELIST#dyn_spectral_as83_nml"
    ! for details about a NAMELIST group.
    !
    use dc_trace, only: BeginSub, EndSub
    use dc_string, only: PutLine, Printf, Split, StrInclude, StoA, JoinChar
    use dc_types, only: STRING, DP, STDOUT
    use dc_present, only: present_and_not_empty, present_and_true
    use dc_message, only: MessageNotify
    use dc_date, only: DCDiffTimeCreate, EvalSec, EvalByUnit
    use dc_error, only: StoreError, DC_NOERR, DC_EALREADYINIT, &
      & DC_EARGLACK, DC_ENEGATIVE, DC_ENOFILEREAD, USR_ERRNO
    use dyn_spectral, only: Create, &
      & GetCoriolis, GetSpectralCoeff, GetDiffCoeff, GetPhis, GetAxes
    use dyn_as83, only: Create, GetAxes
    use dc_hash, only: HASH, DCHashPut, DCHashRewind, DCHashNext, DCHashDelete
    use gt4_history_nmlinfo, only: HstNmlInfoCreate, HstNmlInfoAdd, &
      & HstNmlInfoEndDefine, HstNmlInfoPutLine
    use gt4_history, only: GT_HISTORY, &
      & HistoryCreate, HistoryAddVariable, HistoryPut, &
      & HistoryAddAttr
    implicit none
    type(DYNSPAS83), intent(inout):: dyn_sp_as
    integer, intent(in):: nmax ! ȿ. 
                               ! Maximum truncated wavenumber
    integer, intent(in):: imax ! ٳʻ. 
                               ! Number of grid points in longitude
    integer, intent(in):: jmax ! ٳʻ. 
                               ! Number of grid points in latitude
    integer, intent(in):: kmax ! ľؿ. 
                               ! Number of vertical level
    real(DP), intent(in):: PI        ! $ \pi $ .    ߼Ψ.         Circular constant
    real(DP), intent(in):: RPlanet   ! $ a $ .      Ⱦ.       Radius of planet
    real(DP), intent(in):: Omega     ! $ \Omega $ . ž®.     Angular velocity
!!$    real(DP), intent(in):: Grav      ! $ g $ .      ϲ®.     Gravitational acceleration
    real(DP), intent(in):: Cp        ! $ C_p $ .    絤갵Ǯ.   Specific heat of air at constant pressure
    real(DP), intent(in):: RAir      ! $ R $ .      絤.   Gas constant of air
    real(DP), intent(in):: EpsV      ! $ \epsilon_v $ .        ʬ. Molecular weight ratio of water vapor
    integer, intent(in):: VisOrder   ! ĶǴμ.  Order of hyper-viscosity
    real(DP), intent(in):: EFoldTime ! ȿФ e-folding time. E-folding time for maximum wavenumber
    real(DP), intent(in):: DelTime ! $ \Delta t $ . ॹƥå. Time step
    real(DP), intent(in), optional:: xy_Phis (0:imax-1, 0:jmax-1)
                              ! $ \Phi_s $ . ɽݥƥ󥷥. 
                              ! Surface geo-potential
    integer, intent(in), optional:: openmp_threads
                              ! OPENMP Ǥκ祹åɿ.
                              ! openmp_threads  1 礭ͤꤹ 
                              ! ISPACK[http://www.gfd-dennou.org/library/ispack/] 
                              ! εĴȡѴ OPENMP ׻
                              ! 롼Ѥ. ׻¹Ԥˤ,
                              ! ¹Ի˴Ķѿ OMP_NUM_THREADS 
                              !  openmp_threads ʲοꤹ
                              ! Υƥ˱ɬפȤʤ.
                              !
                              ! openmp_threads  1 礭ͤ
                              ! ꤷʤ׻롼ϸƤФʤ.
                              !
                              ! Maximum number of threads in OPENMP. 
                              ! If integer more than 1 is given, 
                              ! spherical harmonics transration subroutines 
                              ! with OPENMP parallel computation in 
                              ! ISPACK[http://www.gfd-dennou.org/library/ispack/] 
                              ! is used. In practice, specify environment 
                              ! variable (OMP_NUM_THREADS etc.) for parallel
                              ! computation.
                              !
                              ! If number less than 2 is given, 
                              ! subroutines of parallel computation 
                              ! is not called.
    real(DP), intent(in), optional:: r_SigmaSet (:)
                              ! ǥեȤǤ, kmax ͤ˱,
                              ! ưŪ $ \sigma $ ٥ (Ⱦ) 
                              ! ꤵ (kmax ͤΤ).
                              ! $ \sigma $ ٥ (Ⱦ). 
                              ! Ūꤹɬפ, 
                              ! ΰͿ.
                              !
                              ! By default, half $ \sigma $ level is 
                              ! specified automatically according to
                              ! the value of kmax (only the value is 
                              ! same as particular values).
                              ! If the half $ \sigma $ level is specified
                              ! manually, give this argument.
    logical, intent(in), optional :: wa_module_initialized
                              ! wa_module (SPMODEL 饤֥) ե饰. 
                              ! SPMODEL 饤֥ wa_module  
                              ! 
                              ! ˥ץ 
                              ! wa_Initial ˤäƽƤ, 
                              !  wa_Initial Ƥ֤ȥ顼ޤ.
                              ! ΰ .true. Ϳ뤳Ȥ, 
                              ! wa_Initial ƤФʤ褦ˤޤ.
                              ! 
                              ! "wa_module" (SPMODEL library) 
                              ! initialization flag. 
                              ! 
                              ! When "wa_module" (SPMODEL library) 
                              ! is initialized by
                              ! "wa_Initial" already, second "wa_Initial"
                              ! causes an error.
                              ! If .true. is specified to this argument,
                              ! "wa_Initial" is not called.
                              ! 
    character(*), intent(in), optional:: time_integration_scheme
                              ! ʬˡ. 
                              ! ʲˡǽ. 
                              !
                              ! Time integration scheme. 
                              ! Available schemes are as follows.
                              ! 
                              ! * "Semi-implicit"
                              ! * "Explicit"
                              ! 
    real, intent(in), optional:: current_time_value
                              ! ߻ο. Numerical value of current time
    character(*), intent(in), optional:: current_time_unit
                              ! ߻ñ. Unit of current time
    character(*), intent(in), optional:: history_varlist
                              ! ҥȥǡνѿꥹ. 
                              ! ޤǶڤä¤٤. 
                              ! (: "Data1,Data2" ). 
                              ! 
                              ! List of variables output to history data. 
                              ! Delimiter is comma. 
                              ! (exp. "Data1,Data2" ). 
                              ! 
    real, intent(in), optional:: history_interval_value
                              ! ҥȥǡνϴֳ֤ο. 
                              ! Numerical value for interval of history data output
    character(*), intent(in), optional:: history_interval_unit
                              ! ҥȥǡνϴֳ֤ñ. 
                              ! Unit for interval of history data output
    character(*), intent(in), optional:: history_precision
                              ! ҥȥǡ. 
                              ! Precision of history data
    character(*), intent(in), optional:: history_fileprefix
                              ! ҥȥǡΥե̾Ƭ. 
                              ! Prefix of history data filenames
    character(*), intent(in), optional :: nmlfile
                              ! NAMELIST ե̾. 
                              ! ΰ˶ʸʳͿ, 
                              ! ꤵ줿ե뤫 
                              ! NAMELIST ѿɤ߹ߤޤ. 
                              ! եɤ߹ʤˤϥ顼
                              ! ޤ.
                              !
                              ! NAMELIST ѿξܺ٤˴ؤƤ 
                              ! NAMELIST#dyn_spectral_as83_nml 
                              ! 򻲾ȤƤ. 
                              !
                              ! NAMELIST file name. 
                              ! If nonnull character is specified to
                              ! this argument, 
                              ! NAMELIST group name is loaded from the 
                              ! file. 
                              ! If the file can not be read, 
                              ! an error occurs.
                              ! 
                              ! See "NAMELIST#dyn_spectral_as83_nml" 
                              ! for details about a NAMELIST group.
                              ! 
    logical, intent(out), optional:: err
                              ! 㳰ѥե饰.
                              ! ǥեȤǤ, μ³ǥ顼
                              ! , ץ϶λޤ.
                              !  *err* Ϳ,
                              ! ץ϶λ, 
                              ! *err*  .true. ޤ.
                              !
                              ! Exception handling flag. 
                              ! By default, when error occur in 
                              ! this procedure, the program aborts. 
                              ! If this *err* argument is given, 
                              ! .true. is substituted to *err* and 
                              ! the program does not abort. 

    !-----------------------------------
    !  ҥȥեؤΥǡ
    !  Configure the settings for history data output
    character(STRING):: name = ''
                              ! ѿ̾. Variable identifier
    character(STRING):: longname = ''
                              ! ѿεŪ̾. Descriptive name of variables
    character(STRING), allocatable:: dims(:)
                              ! ɸ̾. Name of axes
    character(STRING):: units = ''
                              ! ñ. Units
    type(GT_HISTORY), pointer:: gthist =>null()
                              ! gt4_history ⥸塼ѹ¤. 
                              ! Derived type for "gt4_history" module
    character(TOKEN):: precision
                              ! ҥȥǡ. 
                              ! Precision of history data
    logical:: average
                              ! ϥǡʿѲե饰. 
                              ! Flag for average of output data
    type(HASH):: registered_varnames
                              ! Υ⥸塼뤫Ϥѿ̾Υꥹ.
                              ! 
                              ! List of names of variables output 
                              ! from this module. 
    logical:: end

    !-----------------------------------
    !  ɸѿ
    !  Coordinate variables
    real(DP):: x_Lon (0:imax-1)
                              ! . Longitude
    real(DP):: y_Lat (0:jmax-1)
                              ! . Latitude
    real(DP):: x_Lon_Weight(0:imax-1)
                              ! ٺɸŤ. 
                              ! Weight of longitude
    real(DP):: y_Lat_Weight(0:jmax-1)
                              ! ٺɸŤ. 
                              ! Weight of latitude
    real(DP):: z_Sigma (0:kmax-1)
                              ! $ \sigma $ ٥ (). 
                              ! Full $ \sigma $ level
    real(DP):: r_Sigma (0:kmax)
                              ! $ \sigma $ ٥ (Ⱦ). 
                              ! Half $ \sigma $ level


    !-----------------------------------
    !  ѿ
    !  Work variables
    real(DP):: xy_Cori (0:imax-1, 0:jmax-1)
                              ! $ f\equiv 2\Omega\sin\varphi $ . 
                              ! ꥪѥ᡼. Coriolis parameter
    integer:: nmo (1:2, 0:nmax, 0:nmax)
                              ! ڥȥź. 
                              ! Spectral subscript expression
    real(DP):: wz_rn ((nmax+1)**2, 0:kmax-1)
                              ! $ -n \times (n+1) $ . ץ饷η. 
                              ! Laplacian coefficient
    real(DP):: wz_DiffVorDiv ((nmax+1)**2, 0:kmax-1)
                              ! $ -K_{HD} [(-1)^{N_D/2}\nabla^{N_D}- (2/a^2)^{N_D/2}] $ . 
                              ! ư̿ʿȻ. 
                              ! Coefficient of horizontal momentum diffusion
    real(DP):: wz_DiffTherm ((nmax+1)**2, 0:kmax-1)
                              ! $ -(-1)^{N_D/2}K_{HD}\nabla^{N_D} $ . 
                              ! Ǯ, ʿȻ. 
                              ! Coefficient of horizontal thermal and water diffusion
    real(DP):: w_Phis ((nmax+1)**2)
                              ! $ \Phi_s $ . ɽݥƥ󥷥. 
                              ! Surface geo-potential

    integer:: stat
    character(STRING):: cause_c
    character(*), parameter:: subname = 'DynSpAsCreate'
  continue
    call BeginSub(subname, version)
    stat = DC_NOERR
    cause_c = ''

    !-----------------------------------------------------------------
    !  Υå
    !  Check initialization
    !-----------------------------------------------------------------
    if (dyn_sp_as % initialized) then
      stat = DC_EALREADYINIT
      cause_c = 'DYNSPAS83'
      goto 999
    end if

    !-----------------------------------------------------------------
    !  Υå
    !  Validation of arguments
    !-----------------------------------------------------------------
    if (nmax < 1) then
      stat = DC_ENEGATIVE
      cause_c = 'nmax'
      goto 999
    end if
    if (imax < 1) then
      stat = DC_ENEGATIVE
      cause_c = 'imax'
      goto 999
    end if
    if (jmax < 1) then
      stat = DC_ENEGATIVE
      cause_c = 'jmax'
      goto 999
    end if
    if (kmax < 1) then
      stat = DC_ENEGATIVE
      cause_c = 'kmax'
      goto 999
    end if

    !-----------------------------------------------------------------
    !  ȿʻ
    !  Configure wave number and grid point
    !-----------------------------------------------------------------
    dyn_sp_as % nmax = nmax
    dyn_sp_as % imax = imax
    dyn_sp_as % jmax = jmax
    dyn_sp_as % kmax = kmax

    !-----------------------------------------------------------------
    !  
    !  Time control
    !-----------------------------------------------------------------
    if ( present(current_time_value) .and. present(current_time_unit) ) then
      call DCDiffTimeCreate( &
        & diff = dyn_sp_as % current_time, &      ! (out)
        & value = real(current_time_value, DP), & ! (in)
        & unit = current_time_unit )              ! (in)
    else
      call DCDiffTimeCreate( &
        & diff = dyn_sp_as % current_time, & ! (out)
        & value = 0.0_DP, &    ! (in)
        & unit = 'sec' )       ! (in)
    end if

    call DCDiffTimeCreate( &
      & diff = dyn_sp_as % delta_time, & ! (out)
      & value = DelTime, &             ! (in)
      & unit = 'sec' )                 ! (in)

    !-----------------------------------------------------------------
    !  ڥȥˡ
    !  Configure the settings for spectral method
    !-----------------------------------------------------------------
    call Create( dyn_sp = dyn_sp_as % dyn_sp, &                   ! (inout)
      & nmax = nmax, imax = imax, jmax = jmax, kmax = kmax, &     ! (in)
      & PI = PI, RPlanet = RPlanet, Omega = Omega, Cp = Cp, &     ! (in)
      & VisOrder = VisOrder, EFoldTime = EFoldTime, &             ! (in)
      & DelTime = DelTime, &                                      ! (in)
      & xy_Phis = xy_Phis, &                                      ! (in)
      & openmp_threads = openmp_threads, &                        ! (in)
      & wa_module_initialized = wa_module_initialized, &          ! (in)
      & nmlfile = nmlfile )                                       ! (in)

    !-----------------------------------------------------------------
    !  Arakawa and Suarez (1983) 
    !  Configure the settings for Arakawa and Suarez (1983)
    !-----------------------------------------------------------------

    !---------------------------------------------
    !  ꥪѥ᡼μ
    !  Get Coriolis parameter
    call GetCoriolis( dyn_sp = dyn_sp_as % dyn_sp, &  ! (inout)
      & xy_Cori = xy_Cori ) ! (out)

    !---------------------------------------------
    !  ڥȥ뷸μ
    !  Get spectral coefficient
    call GetSpectralCoeff( dyn_sp = dyn_sp_as % dyn_sp, &  ! (inout)
      & nmo = nmo, wz_rn = wz_rn ) ! (out)

    !---------------------------------------------
    !  Ȼμ
    !  Get diffusion coefficient
    call GetDiffCoeff( dyn_sp = dyn_sp_as % dyn_sp, &  ! (inout)
      & wz_DiffVorDiv = wz_DiffVorDiv, wz_DiffTherm = wz_DiffTherm ) ! (out)

    !---------------------------------------------
    !  Ϸǡ (ɽ $ \Phi $ )μ
    !  Get geography data (surface $ \Phi $ )
    call GetPhis( dyn_sp = dyn_sp_as % dyn_sp, & ! (inout)
      & w_Phis = w_Phis ) ! (out)

    call Create( dyn_as = dyn_sp_as % dyn_as, &                   ! (inout)
      & nmax = nmax, imax = imax, jmax = jmax, kmax = kmax, &     ! (in)
      & PI = PI, RPlanet = RPlanet, &                             ! (in)
      & Cp = Cp, RAir = RAir, EpsV = EpsV, &                      ! (in)
      & DelTime = DelTime, &                                      ! (in)
      & xy_Cori = xy_Cori, &                                      ! (in)
      & nmo = nmo, wz_rn = wz_rn, &                               ! (in)
      & wz_DiffVorDiv = wz_DiffVorDiv, &                          ! (in)
      & wz_DiffTherm = wz_DiffTherm, &                            ! (in)
      & r_SigmaSet = r_SigmaSet, w_Phis = w_Phis, &               ! (in)
      & time_integration_scheme = time_integration_scheme, &      ! (in)
      & nmlfile = nmlfile )                                       ! (in)


    !-----------------------------------------------------------------
    !  ٷٺɸͤμ
    !  Get data of latitude and longitude
    !-----------------------------------------------------------------
    call GetAxes( dyn_sp = dyn_sp_as % dyn_sp, &  ! (inout)
      & x_Lon = x_Lon, y_Lat = y_Lat, & ! (out)
      & x_Lon_Weight = x_Lon_Weight, &  ! (out)
      & y_Lat_Weight = y_Lat_Weight )   ! (out)

    !-----------------------------------------------------------------
    !  ľ٥ɸͤμ
    !  Get data of vertical level
    !-----------------------------------------------------------------
    call GetAxes( dyn_as = dyn_sp_as % dyn_as, &  ! (inout)
      & z_Sigma = z_Sigma, r_Sigma = r_Sigma ) ! (out)

    !-----------------------------------------------------------------
    !  ҥȥ꡼ǡ˴ؤ
    !  Time control for history data
    !-----------------------------------------------------------------
    if ( present(current_time_value) .and. present(current_time_unit) ) then
      call DCDiffTimeCreate( &
        & diff = dyn_sp_as % current_time, &  ! (out)
        & value = real( current_time_value, DP ), & ! (in)
        & unit = current_time_unit )                ! (in)
    else
      call DCDiffTimeCreate( &
        & diff = dyn_sp_as % current_time, & ! (out)
        & value = 0.0_DP, &    ! (in)
        & unit = 'sec' )       ! (in)
    end if

    call DCDiffTimeCreate( &
      & diff = dyn_sp_as % delta_time, & ! (out)
      & value = DelTime, &             ! (in)
      & unit = 'sec' )                 ! (in)

    !-----------------------------------------------------------------
    !  ҥȥեؤΥǡ
    !  Configure the settings for history data output
    !-----------------------------------------------------------------

    !-------------------------
    !  ǥե
    !  Default values
    call HstNmlInfoCreate( gthstnml = dyn_sp_as % gthstnml ) ! (inout)

    !-------------------------
    !  ץʥ
    !  Values from optional arguments
    call HstNmlInfoAdd( &
      & gthstnml = dyn_sp_as % gthstnml, &         ! (inout)
      & name = '', &                               ! (in)
      & interval_value = history_interval_value, & ! (in)
      & interval_unit = history_interval_unit, &   ! (in)
      & precision = history_precision, &           ! (in)
      & average = .false., &                       ! (in)
      & fileprefix = history_fileprefix )          ! (in)

    if ( present(history_varlist) ) then
      call HstNmlInfoAdd( &
        & gthstnml = dyn_sp_as % gthstnml, & ! (inout)
        & name = history_varlist )                 ! (in)
    end if

    !-----------------------------------------------------------------
    !  NAMELIST ͤɤ߹
    !  Load values from NAMELIST
    !-----------------------------------------------------------------
    if ( present_and_not_empty(nmlfile) ) then
      call MessageNotify( 'M', subname, &
        & 'Loading NAMELIST file "%c" ...', &
        & c1 = trim(nmlfile) )
      call NmlRead ( nmlfile = nmlfile, &          ! (in)
        & gthstnml    = dyn_sp_as % gthstnml, &  ! (inout)
        & err = err )                              ! (out)
      if ( present_and_true(err) ) then
        call MessageNotify( 'W', subname, &
          & '"%c" can not be read.', &
          & c1 = trim(nmlfile) )
        stat = DC_ENOFILEREAD
        cause_c = nmlfile
        goto 999
      end if
    end if

    call HstNmlInfoEndDefine( &
      & gthstnml = dyn_sp_as % gthstnml, & ! (inout)
      & err = err )                              ! (out)
    if ( present_and_true( err ) ) then
      stat = USR_ERRNO
      goto 999
    end if


    !-----------------------------------------------------------------
    !  ǡϤν
    !  Initialize data output
    !-----------------------------------------------------------------

    !-------------------------
    !  xyr_SigmaDot ν
    !  Configure the settings for "xyr_SigmaDot" output
    name = 'SigmaDot'
    longname = 'sigma-vertical velocity'
    units = '1 s-1'
    allocate( dims(4) )
    dims = StoA( 'lon', 'lat', 'sigm', 'time' )

    ! ϥեν.
    !   * gthist (gt4_history#GT_HISTORY) ꤵ.
    ! Initialize output file.
    !   * "gthist" (gt4_history#GT_HISTORY) is configured.
    call output_init  ! ֥롼. This is internal subroutine

    ! °ղäʤɤԤˤϰʲΤ褦ˤ. 
    ! Describe codes as follows in order to add attributes etc.
!!$    if ( associated( gthist ) ) then
!!$      call HistoryAddAttr( &
!!$        & history = gthist, &                           ! (inout)
!!$        & varname = name, attrname = 'missing_value', & ! (in)
!!$        &   value = 9999.0 )                            ! (in)
!!$    end if
    deallocate( dims )

    !-------------------------
    !  xy_DPiDt ν
    !  Configure the settings for "xy_DPiDt" output
    name = 'DPiDt'
    longname = 'Pi (log Ps) tendency)'
    units = 'Pa s-1'
    allocate( dims(3) )
    dims = StoA( 'lon', 'lat', 'time' )

    ! ϥեν.
    !   * gthist (gt4_history#GT_HISTORY) ꤵ.
    ! Initialize output file.
    !   * "gthist" (gt4_history#GT_HISTORY) is configured.
    call output_init  ! ֥롼. This is internal subroutine
    deallocate( dims )

    !-----------------------------------------------------------------
    !  Υ⥸塼뤫Ϥѿ̾ΥꥹȤɽ
    !  Print list of names of variables output from this module
    !-----------------------------------------------------------------
    call Printf( STDOUT, &
      & ' *** MESSAGE *** +---- "%c" module output varnames list -----', &
      & c1 = 'dcmodel_sample_code' )
    call DCHashRewind( hashv = registered_varnames ) ! (inout)
    do
      call DCHashNext( hashv = registered_varnames, & ! (inout)
        & key = name, value = longname, end = end )   ! (out)
      if ( end ) exit
      call Printf( STDOUT, &
        & ' *** MESSAGE *** |      "%c" (%c)', &
        & c1 = trim(name), c2 = trim(longname) )
    enddo
    call DCHashDelete( hashv = registered_varnames ) ! (inout)
    call Printf( STDOUT, &
      & ' *** MESSAGE *** `----------------------------------------' )

    !-----------------------------------------------------------------
    !  λ, 㳰
    !  Termination and Exception handling
    !-----------------------------------------------------------------
    dyn_sp_as % initialized = .true.
999 continue
    call StoreError(stat, subname, err, cause_c)
    call EndSub(subname)

  contains

    subroutine output_init
      !
      ! ѿ *name* ˴ؤƽϥեνԤޤ. 
      ! ϥե̾ϴֳ֤ʤɤξ dyn_sp_as % gthstnml 
      ! Фޤ. 
      ! 
      ! ѿ *name* ˴ؤƽϤԤˤ, 
      ! *gthist* ˽ե gt4_history#GT_HISTORY
      ! ѿ礵ޤ. Ǥʤ, *gthist* ֤ˤޤ. 
      !
      ! ޤ, ϥǡ٤ precision , 
      ! ϥǡʿѲβݤ average ꤷޤ. 
      ! 
      ! ɸϤɽѿꥹ *registered_varnames* 
      ! *name*, *longname*, *dims*, *units* Ͽޤ. 
      !
      ! An output file is initialized for a variable *name*. 
      ! Information such as the output filename and output intervals 
      ! is taken out of "dyn_sp_as % gthstnml". 
      !
      ! When output is done for the variable *name*, *gthist* is 
      ! associated with the "gt4_history#GT_HISTORY" variable of 
      ! the output file. Otherwise, *gthist* is nullified. 
      !
      ! Moreover, the accuracy of output data is set to *precision*, and 
      ! right or wrong of averaging the output data is set to *average*. 
      !
      ! *name*, *longname*, *dims*, *units* are registered to 
      ! a list of variables *registered_varnames* that is printed to 
      ! standard output. 
      !
      use dc_date, only: DCDiffTimeCreate, EvalSec, EvalByUnit
      use gt4_history_nmlinfo, only: GTHST_NMLINFO, &
        & HstNmlInfoInitialized, HstNmlInfoInquire, &
        & HstNmlInfoOutputValid, HstNmlInfoAssocGtHist, HstNmlInfoPutLine
      use gt4_history, only: GT_HISTORY, &
        & HistoryCreate, HistoryAddVariable, HistoryPut, &
        & HistoryAddAttr, HistoryInitialized

      !-----------------------------------
      !  ѿ
      !  Work variables
      character(STRING):: file
                                ! ҥȥǡΥե̾. 
                                ! History data filenames
      character(STRING):: dims_str
                                ! ɸΥꥹ. 
                                ! List of axes
      real:: interval_value
                                ! ҥȥǡνϴֳ֤ο. 
                                ! Numerical value for interval of history data output
      character(TOKEN):: interval_unit
                                ! ҥȥǡνϴֳ֤ñ. 
                                ! Unit for interval of history data output
      real(DP), parameter:: PI = 3.1415926535897930_DP
                                ! $ \pi $ . ߼Ψ. Circular constant
    continue
      !-----------------------------------------------------------------
      !  ɸϤɽѿϿ
      !  Register a variable name for print to standard output
      !-----------------------------------------------------------------
      if ( allocated(dims) ) then
        dims_str = JoinChar( dims, ',' )
      else
        dims_str = ''
      end if
      call DCHashPut( hashv = registered_varnames, &     ! (inout)
        & key = name, &                                  ! (in)
        & value = trim( longname ) // ' [' // &
        &         trim( units ) // '] {' // &
        &         trim( dims_str ) // '}' ) ! (in)

      !-----------------------------------------------------------------
      !  ѿν
      !  Initialize variable
      !-----------------------------------------------------------------
      nullify( gthist )
      precision = 'float'
      average = .false.

      !-----------------------------------------------------------------
      !  Ϥͭɤǧ
      !  Confirm whether the output is effective
      !-----------------------------------------------------------------
      if ( .not. HstNmlInfoOutputValid( dyn_sp_as % gthstnml, name ) ) then
        return
      end if

      !-----------------------------------------------------------------
      !  GT_HISTORY ѿμ
      !  Get "GT_HISTORY" variable
      !-----------------------------------------------------------------
      call HstNmlInfoAssocGtHist( &
        & gthstnml = dyn_sp_as % gthstnml, & ! (in)
        & name = name, &                       ! (in)
        & history = gthist, &                  ! (out)
        & err = err )                          ! (out)
      if ( present_and_true( err ) ) return

      call HstNmlInfoInquire( &
        & gthstnml = dyn_sp_as % gthstnml, & ! (in)
        & name = name, &                       ! (in)
        & precision = precision, &             ! (out)
        & average = average, &                 ! (out)
        & err = err )                          ! (out)
      if ( present_and_true( err ) ) return

      !-----------------------------------------------------------------
      !  GT_HISTORY ѿνγǧ
      !  Check initialization of "GT_HISTORY" variable
      !-----------------------------------------------------------------
      if ( HistoryInitialized( gthist ) ) then

        !---------------------------------------------------------------
        !  HistoryAddVariable ˤѿ
        !  A variable is created by "HistoryAddVariable"
        !---------------------------------------------------------------
        call HistoryAddVariable( &
          &  history = gthist, &                       ! (inout)
          &  varname = name,         dims = dims, &    ! (in)
          & longname = longname,    units = units, &   ! (in)
          &    xtype = precision, average = average )  ! (in)
        return
      end if

      !-----------------------------------------------------------------
      !  HistoryCreate Τͤμ
      !  Get the settings for "HistoryCreate"
      !-----------------------------------------------------------------
      call HstNmlInfoInquire( &
        & gthstnml = dyn_sp_as % gthstnml, & ! (in)
        & name = name, &                       ! (in)
        & file = file, &                       ! (out)
        & interval_unit = interval_unit, &     ! (out)
        & interval_value = interval_value, &   ! (out)
        & err = err )                          ! (out)
      if ( present_and_true( err ) ) return

      !-----------------------------------------------------------------
      !  HistoryCreate ˤե
      !  Files are created by "HistoryCreate"
      !-----------------------------------------------------------------
      call HistoryCreate( &
        & history = gthist, &   ! (out)
        & file = file, &        ! (in)
        & title = 'Dynamical core calculation', &  ! (in)
        & source = 'dcpam4 : ' // trim(version), &               ! (in)
        & institution = 'GFD Dennou Club', &                     ! (in)
        & dims = StoA('lon', 'lat', 'sig', 'sigm', 'time'), &    ! (in)
        & dimsizes = (/ dyn_sp_as % imax, dyn_sp_as % jmax, &
        &               dyn_sp_as % kmax, dyn_sp_as % kmax+1, 0 /), & ! (in)
        & longnames = StoA('longitude', 'latitude', &
        &                  'sigma at layer midpoints', &
        &                  'sigma at layer end-points (half level)', &
        &                  'time'), &                            ! (in)
        & units = StoA( 'degree_east', 'degree_north', &
        &               '1', '1', interval_unit ), &             ! (in)
        & origin = real( EvalbyUnit( dyn_sp_as % current_time, &
        &                            interval_unit) ), &         ! (in)
        & interval = interval_value, &                           ! (in)
        & err = err )                                            ! (out)
      if ( present_and_true( err ) ) then
        nullify( gthist )
        return
      end if

      call HistoryAddAttr( &
        & history = gthist, &                            ! (inout)
        & varname = 'lon', attrname = 'standard_name', & ! (in)
        & value = 'longitude' )                          ! (in)
      call HistoryAddAttr( &
        & history = gthist, &                            ! (inout)
        & varname = 'lat', attrname = 'standard_name', & ! (in)
        & value = 'latitude' )                           ! (in)
      call HistoryAddAttr( &
        & history = gthist, &                              ! (inout)
        & varname = 'sig', attrname = 'standard_name', &   ! (in)
        & value = 'atmosphere_sigma_coordinate' )          ! (in)
      call HistoryAddAttr( &
        & history = gthist, &                              ! (inout)
        & varname = 'sigm', attrname = 'standard_name', &  ! (in)
        & value = 'atmosphere_sigma_coordinate' )          ! (in)
      call HistoryAddAttr( &
        & history = gthist, &                             ! (inout)
        & varname = 'time', attrname = 'standard_name', & ! (in)
        & value = 'time' )                                ! (in)
      call HistoryAddAttr( &
        & history = gthist, &                              ! (inout)
        & varname = 'sig', attrname = 'positive', &        ! (in)
        & value = 'down' )                                 ! (in)
      call HistoryAddAttr( &
        & history = gthist, &                              ! (inout)
        & varname = 'sigm', attrname = 'positive', &       ! (in)
        & value = 'down' )                                 ! (in)

      call HistoryPut( &
        & history = gthist, &             ! (inout)
        & varname = 'lon', &              ! (in)
        & array = x_Lon / PI * 180.0_DP ) ! (in)
      call HistoryPut( &
        & history = gthist, &             ! (inout)
        & varname = 'lat', &              ! (in)
        & array = y_Lat / PI * 180.0_DP ) ! (in)
      call HistoryPut( &
        & history = gthist, & ! (inout)
        & varname = 'sig', &  ! (in)
        & array = z_Sigma )   ! (in)
      call HistoryPut( & 
        & history = gthist, & ! (inout)
        & varname = 'sigm', & ! (in)
        & array = r_Sigma )   ! (in)

      call HistoryAddVariable( &
        & history = gthist, &                                 ! (inout)
        & varname = 'lon_weight', &                           ! (in)
        & dims = StoA('lon'), &                               ! (in)
        & longname = 'weight for integration in longitude', & ! (in)
        & units = 'radian', xtype = 'double' )                ! (in)
      call HistoryAddAttr( &
        & history = gthist, &                                 ! (inout)
        & varname = 'lon', attrname = 'gt_calc_weight', &     ! (in)
        & value = 'lon_weight' )                              ! (in)
      call HistoryPut( &
        & history = gthist, &                     ! (inout)
        & varname = 'lon_weight', &               ! (in)
        & array = x_Lon_Weight )                  ! (in)

      call HistoryAddVariable( &
        & history = gthist, &                                 ! (inout)
        & varname = 'lat_weight', &                           ! (in)
        & dims = StoA('lat'), &                               ! (in)
        & longname = 'weight for integration in latitude', &  ! (in)
        & units = 'radian', xtype = 'double' )                ! (in)
      call HistoryAddAttr( &
        & history = gthist, &                                 ! (inout)
        & varname = 'lat', attrname = 'gt_calc_weight', &     ! (in)
        & value = 'lat_weight' )                              ! (in)
      call HistoryPut( &
        & history = gthist, &                     ! (inout)
        & varname = 'lat_weight', &               ! (in)
        & array = y_Lat_Weight )                  ! (in)

      !-----------------------------------------------------------------
      !  HistoryAddVariable ˤѿ
      !  A variable is created by "HistoryAddVariable"
      !-----------------------------------------------------------------
      if ( HistoryInitialized( gthist ) ) then
        call HistoryAddVariable( &
          &  history = gthist, &                       ! (inout)
          &  varname = name,         dims = dims, &    ! (in)
          & longname = longname,    units = units, &   ! (in)
          &    xtype = precision, average = average )  ! (in)
      else
        nullify( gthist )
      end if

    end subroutine output_init

  end subroutine DynSpAsCreate

  subroutine DynSpAsDynamics( dyn_sp_as, &
    & xyz_VorB, xyz_DivB, xyz_TempB, xyz_QVapB, xy_PsB, &
    & xyz_VorN, xyz_DivN, xyz_TempN, xyz_QVapN, xy_PsN, &
    & xyz_VorA, xyz_DivA, xyz_TempA, xyz_QVapA, xy_PsA, &
    & xyz_DVorDt, xyz_DDivDt, xyz_DTempDt, xyz_DQVapDt, &
    & historyput_flag, &
    & err )
    !
    ! ϳزα黻Ԥ, Ϳ줿 $ t-\Delta t $  $ t $ 
    !  (xyz_VorB, xyz_VorN), ȯ (xyz_DivB, xyz_DivN), 
    !  (xyz_TempB, xyz_TempN), 漾 (xyz_QVapB, xyz_QVapN), 
    ! ɽ̵ (xyz_PsB, xyz_PsN) , 
    ! $ t+\Delta t $ 
    !  (xyz_VorA), ȯ (xyz_DivA),  (xyz_TempA), 
    ! 漾 (xyz_QVapA), ɽ̵ (xyz_PsA) ֤ޤ.
    !
    ! ̤ʪץˤ뱲, ȯ, , 漾Ѳ, 
    ! ϳزѲ­ƼΥƥåפ׻ˤ, 
    ! Ѳ xyz_DVorDt, xyz_DDivDt, xyz_DTempDt, xyz_DQVapDt
    ! ͿƤ. 
    !
    ! ʬˡˤϥ꡼ץեåѤƤޤ.
    ! ǥեȤǤ, $ \Delta t $ 礭Ȥ뤿, ȹ 
    ! ߥץꥷåˡŬѤƤޤ.
    ! , Create  *time_integration_scheme* 
    ! "Explicit" ꤷˤϽȹ⥨ץꥷåˡˤä
    ! 򤭤ޤ.
    !
    ! ʤ, Ϳ줿 *dyn_sp_as*  Create ˤäƽ
    ! Ƥʤ, ץϥ顼ȯޤ.
    !
    ! Calculating dynamical core, 
    ! from vorticity (xyz_VorB, xyz_VorN), divergence (xyz_DivB, xyz_DivN), 
    ! temperature (xyz_TempB, xyz_TempN), 
    ! specific humidity (xyz_QVapB, xyz_QVapN), 
    ! surface pressure (xyz_PsB, xyz_PsN) at $ t-\Delta t $ and $ t $, 
    ! vorticity (xyz_VorA), divergence (xyz_DivA), temperature (xyz_TempA), 
    ! specific humidity (xyz_QVapA), surface pressure (xyz_PsA) 
    ! are returned.
    !
    ! In order to add tendencies of vorticity, divergence, 
    ! temperature and specific humidity by another physical process to 
    ! tendencies of this dynamical process 
    ! and to calculate the values at next time step, 
    ! give these tendencies to 
    ! "xyz_DVorDt", "xyz_DDivDt", "xyz_DTempDt", "xyz_DQVapDt"
    !
    ! Leap-frog scheme is used as time integration. 
    ! By default, semi-implicit scheme is applied to gravitational terms 
    ! for extension of $ \Delta t $ . 
    ! If "Explicit" is specified to *time_integration_scheme* in "Create", 
    ! explicit scheme is applied to gravitational terms as well as 
    ! other terms.
    !
    ! If *dyn_sp_as* is not initialized by "Create" yet,
    ! error is occurred.
    !
    use dc_trace, only: BeginSub, EndSub
    use dc_types, only: STRING, STDOUT, DP, TOKEN
    use dc_string, only: LChar, PutLine, Printf, Split, StrInclude, StoA, JoinChar
    use dc_present, only: present_and_true
    use dc_date, only: mod, operator(+), operator(==), EvalbyUnit
    use dcpam_error, only: StoreError, DC_NOERR, DCPAM_ENOTINIT
    use dyn_spectral, only: GradPi, VorDiv2UV, Tendency, TendencyExplicit, &
      & Spectral2Grid, Grid2Spectral, &
      & DiffusionCorrectTemp, DiffusionVorDiv
    use dyn_as83, only: NonLinearOnGrid, TimeIntegration, Inquire, &
      & WTplusGPiOnGrid, HDivOnGrid
    use gt4_history_nmlinfo, only: HstNmlInfoInquire, &
      & HstNmlInfoOutputValid, HstNmlInfoAssocGtHist, HstNmlInfoPutLine
    use gt4_history, only: HistoryPut, HistoryInitialized
    implicit none
    type(DYNSPAS83), intent(inout):: dyn_sp_as
    real(DP), intent(in):: xyz_VorB (0:dyn_sp_as%imax-1, 0:dyn_sp_as%jmax-1, 0:dyn_sp_as%kmax-1)
                              ! $ \zeta (t-\Delta t) $ . . Vorticity
    real(DP), intent(in):: xyz_DivB (0:dyn_sp_as%imax-1, 0:dyn_sp_as%jmax-1, 0:dyn_sp_as%kmax-1)
                              ! $ D (t-\Delta t) $ .     ȯ. Divergence
    real(DP), intent(in):: xyz_TempB (0:dyn_sp_as%imax-1, 0:dyn_sp_as%jmax-1, 0:dyn_sp_as%kmax-1)
                              ! $ T (t-\Delta t) $ .     . Temperature
    real(DP), intent(in):: xy_PsB (0:dyn_sp_as%imax-1, 0:dyn_sp_as%jmax-1)
                              ! $ P_s (t-\Delta t) $ .   ɽ̵. Surface pressure
    real(DP), intent(in):: xyz_QVapB (0:dyn_sp_as%imax-1, 0:dyn_sp_as%jmax-1, 0:dyn_sp_as%kmax-1)
                              ! $ q (t-\Delta t) $ .     漾. Specific humidity
    real(DP), intent(in):: xyz_VorN (0:dyn_sp_as%imax-1, 0:dyn_sp_as%jmax-1, 0:dyn_sp_as%kmax-1)
                              ! $ \zeta (t) $ . . Vorticity
    real(DP), intent(in):: xyz_DivN (0:dyn_sp_as%imax-1, 0:dyn_sp_as%jmax-1, 0:dyn_sp_as%kmax-1)
                              ! $ D (t) $ .     ȯ. Divergence
    real(DP), intent(in):: xyz_TempN (0:dyn_sp_as%imax-1, 0:dyn_sp_as%jmax-1, 0:dyn_sp_as%kmax-1)
                              ! $ T (t) $ .     . Temperature
    real(DP), intent(in):: xy_PsN (0:dyn_sp_as%imax-1, 0:dyn_sp_as%jmax-1)
                              ! $ P_s (t) $ .   ɽ̵. Surface pressure
    real(DP), intent(in):: xyz_QVapN (0:dyn_sp_as%imax-1, 0:dyn_sp_as%jmax-1, 0:dyn_sp_as%kmax-1)
                              ! $ q (t) $ .     漾. Specific humidity
    real(DP), intent(out):: xyz_VorA (0:dyn_sp_as%imax-1, 0:dyn_sp_as%jmax-1, 0:dyn_sp_as%kmax-1)
                              ! $ \zeta (t+\Delta t) $ . . Vorticity
    real(DP), intent(out):: xyz_DivA (0:dyn_sp_as%imax-1, 0:dyn_sp_as%jmax-1, 0:dyn_sp_as%kmax-1)
                              ! $ D (t+\Delta t) $ .     ȯ. Divergence
    real(DP), intent(out):: xyz_TempA (0:dyn_sp_as%imax-1, 0:dyn_sp_as%jmax-1, 0:dyn_sp_as%kmax-1)
                              ! $ T (t+\Delta t) $ .     . Temperature
    real(DP), intent(out):: xy_PsA (0:dyn_sp_as%imax-1, 0:dyn_sp_as%jmax-1)
                              ! $ P_s (t+\Delta t) $ .   ɽ̵. Surface pressure
    real(DP), intent(out):: xyz_QVapA (0:dyn_sp_as%imax-1, 0:dyn_sp_as%jmax-1, 0:dyn_sp_as%kmax-1)
                              ! $ q (t+\Delta t) $ .     漾. Specific humidity
    real(DP), intent(in), optional:: xyz_DVorDt (0:dyn_sp_as%imax-1, 0:dyn_sp_as%jmax-1, 0:dyn_sp_as%kmax-1)
                              ! $ \DP{\zeta}{t} $ . 
                              ! Ѳ. 
                              ! Vorticity tendency
    real(DP), intent(in), optional:: xyz_DDivDt (0:dyn_sp_as%imax-1, 0:dyn_sp_as%jmax-1, 0:dyn_sp_as%kmax-1)
                              ! $ \DP{D}{t} $ . 
                              ! ȯѲ. 
                              ! Divergence tendency
    real(DP), intent(in), optional:: xyz_DTempDt (0:dyn_sp_as%imax-1, 0:dyn_sp_as%jmax-1, 0:dyn_sp_as%kmax-1)
                              ! $ \DP{T}{t} $ . 
                              ! Ѳ. 
                              ! Temperature tendency
    real(DP), intent(in), optional:: xyz_DQVapDt (0:dyn_sp_as%imax-1, 0:dyn_sp_as%jmax-1, 0:dyn_sp_as%kmax-1)
                              ! $ \DP{q}{t} $ . 
                              ! 漾Ѳ. 
                              ! Specific humidity tendency
    logical, intent(in), optional:: historyput_flag
                              ! ǡϤΥե饰. 
                              ! SetTime ˤäƻŪ
                              ! ꤷˤ, ΰ
                              ! .true. ޤ .false. ꤹ
                              ! ȤǥǡϤΥ󥪥դ
                              ! Ū˻ꤹɬפޤ. 
                              ! ǥեȤ .false. Ǥ.
                              ! 
                              ! Data output flag. 
                              ! When time is specified by "SetTime", 
                              ! explicit specification of data output
                              ! on/off by specifying ".true." or ".false."
                              ! to this argument.
                              ! Default value is ".false.". 
                              ! 
    logical, intent(out), optional:: err
                              ! 㳰ѥե饰.
                              ! ǥեȤǤ, μ³ǥ顼
                              ! , ץ϶λޤ.
                              !  *err* Ϳ,
                              ! ץ϶λ, 
                              ! *err*  .true. ޤ.
                              !
                              ! Exception handling flag. 
                              ! By default, when error occur in 
                              ! this procedure, the program aborts. 
                              ! If this *err* argument is given, 
                              ! .true. is substituted to *err* and 
                              ! the program does not abort. 

    !-----------------------------------
    !  ѿ
    !  Work variables
    real(DP):: xy_GradLonPiN (0:dyn_sp_as%imax-1, 0:dyn_sp_as%jmax-1)
                              ! $ \DD{\pi}{x} (t) $
    real(DP):: xy_GradLatPiN (0:dyn_sp_as%imax-1, 0:dyn_sp_as%jmax-1)
                              ! $ \DD{\pi}{y} (t) $
    real(DP):: xyz_UN (0:dyn_sp_as%imax-1, 0:dyn_sp_as%jmax-1, 0:dyn_sp_as%kmax-1)
                              ! $ U (t) $ . ®. Zonal wind.
    real(DP):: xyz_VN (0:dyn_sp_as%imax-1, 0:dyn_sp_as%jmax-1, 0:dyn_sp_as%kmax-1)
                              ! $ V (t) $ . ®. Meridional wind.
    real(DP):: xyz_UA (0:dyn_sp_as%imax-1, 0:dyn_sp_as%jmax-1, 0:dyn_sp_as%kmax-1)
                 ! $ U (t+\Delta t) $ . ®. Zonal wind
    real(DP):: xyz_VA (0:dyn_sp_as%imax-1, 0:dyn_sp_as%jmax-1, 0:dyn_sp_as%kmax-1)
                              ! $ V (t+\Delta t) $ . ®. Meridional wind
    real(DP):: xyz_UAdvN (0:dyn_sp_as%imax-1, 0:dyn_sp_as%jmax-1, 0:dyn_sp_as%kmax-1)
                              ! $ UA (t) $ . ư̰ή. 
                              ! Zonal advection of momentum
    real(DP):: xyz_VAdvN (0:dyn_sp_as%imax-1, 0:dyn_sp_as%jmax-1, 0:dyn_sp_as%kmax-1)
                              ! $ VA (t) $ . ̱ư̰ή. 
                              ! Meridional advection of momentum
    real(DP):: xyz_DTempDtN (0:dyn_sp_as%imax-1, 0:dyn_sp_as%jmax-1, 0:dyn_sp_as%kmax-1)
                              ! $ H (t) $ . ٻѲ. 
                              ! Temperature tendency
    real(DP):: xyz_DQVapDtN (0:dyn_sp_as%imax-1, 0:dyn_sp_as%jmax-1, 0:dyn_sp_as%kmax-1)
                              ! $ R (t) $ . 漾Ѳ. 
                              ! Specific humidity tendency
    real(DP):: xyz_KEN (0:dyn_sp_as%imax-1, 0:dyn_sp_as%jmax-1, 0:dyn_sp_as%kmax-1)
                              ! $ E (t) $ . ưͥ륮. 
                              ! Kinematic energy
    real(DP):: xyz_TempUAdvN (0:dyn_sp_as%imax-1, 0:dyn_sp_as%jmax-1, 0:dyn_sp_as%kmax-1)
                              ! $ UT (t) $ . ή. 
                              ! Zonal advection of temperature
    real(DP):: xyz_TempVAdvN (0:dyn_sp_as%imax-1, 0:dyn_sp_as%jmax-1, 0:dyn_sp_as%kmax-1)
                              ! $ VT (t) $ . ̰ή. 
                              ! Meridional advection of temperature
    real(DP):: xyr_SigmaDotN (0:dyn_sp_as%imax-1, 0:dyn_sp_as%jmax-1, 0:dyn_sp_as%kmax)
                              ! $ \dot{\sigma} $ .
                              ! ľή. Vertical flow
    real(DP):: xy_DPiDtN (0:dyn_sp_as%imax-1, 0:dyn_sp_as%jmax-1)
                              ! $ Z $ . ɽ̵Ѳ. 
                              ! Surface pressure tendency
    real(DP):: xyz_QVapUAdvN (0:dyn_sp_as%imax-1, 0:dyn_sp_as%jmax-1, 0:dyn_sp_as%kmax-1)
                              ! $ Uq (t) $ . 漾ή. 
                              ! Zonal advection of specific humidity
    real(DP):: xyz_QVapVAdvN (0:dyn_sp_as%imax-1, 0:dyn_sp_as%jmax-1, 0:dyn_sp_as%kmax-1)
                              ! $ Vq (t) $ . 漾̰ή. 
                              ! Meridional advection of specific humidity
    real(DP):: wz_DVorDtN ((dyn_sp_as%nmax+1)**2, 0:dyn_sp_as%kmax-1)
                              ! $ \DD{\zeta}{t} (t) $ . Ѳ (ڥȥ). 
                              ! Vorticity tendency (spectral)
    real(DP):: wz_DDivDtN ((dyn_sp_as%nmax+1)**2, 0:dyn_sp_as%kmax-1)
                              ! $ \DD{D}{t} (t) $ . ȯѲ (ڥȥ). 
                              ! Divergence tendency (spectral)
    real(DP):: wz_DTempDtN ((dyn_sp_as%nmax+1)**2, 0:dyn_sp_as%kmax-1)
                              ! $ \DD{T}{t} (t) $ . Ѳ (ڥȥ). 
                              ! Temperature tendency (spectral)
    real(DP):: wz_DQVapDtN ((dyn_sp_as%nmax+1)**2, 0:dyn_sp_as%kmax-1)
                              ! $ \DD{q}{t} (t) $ . 漾Ѳ (ڥȥ). 
                              ! Specific humidity tendency (spectral)
    real(DP):: w_DPiDtN ((dyn_sp_as%nmax+1)**2)
                              ! $ \DD{Ps}{t} (t) $ . ɽ̵Ѳ (ڥȥ). 
                              ! Surface pressure tendency (spectral)
    real(DP):: wz_VorB ((dyn_sp_as%nmax+1)**2, 0:dyn_sp_as%kmax-1)
                              ! $ \zeta (t-\Delta t) $ .  (ڥȥ). 
                              ! Vorticity (spectral)
    real(DP):: wz_DivB ((dyn_sp_as%nmax+1)**2, 0:dyn_sp_as%kmax-1)
                              ! $ D (t-\Delta t) $ . ȯ (ڥȥ). 
                              ! Divergence (spectral)
    real(DP):: wz_TempB ((dyn_sp_as%nmax+1)**2, 0:dyn_sp_as%kmax-1)
                              ! $ T (t-\Delta t) $ .  (ڥȥ). 
                              ! Temperature (spectral)
    real(DP):: w_PiB ((dyn_sp_as%nmax+1)**2)
                              ! $ \pi = \ln P_s (t-\Delta t) $ . ɽ̵ (ڥȥ). 
                              ! Surface pressure (spectral)
    real(DP):: wz_QVapB ((dyn_sp_as%nmax+1)**2, 0:dyn_sp_as%kmax-1)
                              ! $ q (t-\Delta t) $ . 漾 (ڥȥ). 
                              ! Specific humidity (spectral)
    real(DP):: wz_VorA ((dyn_sp_as%nmax+1)**2, 0:dyn_sp_as%kmax-1)
                              ! $ \zeta (t+\Delta t) $ .  (ڥȥ). 
                              ! Vorticity (spectral)
    real(DP):: wz_DivA ((dyn_sp_as%nmax+1)**2, 0:dyn_sp_as%kmax-1)
                              ! $ D (t+\Delta t) $ . ȯ (ڥȥ). 
                              ! Divergence (spectral)
    real(DP):: wz_TempA ((dyn_sp_as%nmax+1)**2, 0:dyn_sp_as%kmax-1)
                              ! $ T (t+\Delta t) $ .  (ڥȥ). 
                              ! Temperature (spectral)
    real(DP):: w_PiA ((dyn_sp_as%nmax+1)**2)
                              ! $ \pi = \ln P_s (t+\Delta t) $ . ɽ̵ (ڥȥ). 
                              ! Surface pressure (spectral)
    real(DP):: wz_QVapA ((dyn_sp_as%nmax+1)**2, 0:dyn_sp_as%kmax-1)
                              ! $ q (t+\Delta t) $ . 漾 (ڥȥ). 
                              ! Specific humidity (spectral)
    real(DP):: wz_VorDiffA ((dyn_sp_as%nmax+1)**2, 0:dyn_sp_as%kmax-1)
                              ! $ \mathscr{D}(\zeta) $ . 
                              ! ư̿ʿȻˤ뱲Ѳ (ڥȥ). 
                              ! Vorticity tendency by 
                              ! horizontal momentum diffusion (spectral)
    real(DP):: wz_DivDiffA ((dyn_sp_as%nmax+1)**2, 0:dyn_sp_as%kmax-1)
                              ! $ \mathscr{D}(D) $ . 
                              ! ư̿ʿȻˤȯѲ (ڥȥ). 
                              ! Divergence tendency by 
                              ! horizontal momentum diffusion (spectral)

    !-----------------------------------
    !  ΥץλѲΤκѿ
    !  Work variables for tendency of external processes

    real(DP):: xyz_DVorDtWork (0:dyn_sp_as%imax-1, 0:dyn_sp_as%jmax-1, 0:dyn_sp_as%kmax-1)
                              ! $ \DP{\zeta}{t} $ . 
                              ! Ѳ. 
                              ! Vorticity tendency
    real(DP):: xyz_DDivDtWork (0:dyn_sp_as%imax-1, 0:dyn_sp_as%jmax-1, 0:dyn_sp_as%kmax-1)
                              ! $ \DP{D}{t} $ . 
                              ! ȯѲ. 
                              ! Divergence tendency
    real(DP):: xyz_DTempDtWork (0:dyn_sp_as%imax-1, 0:dyn_sp_as%jmax-1, 0:dyn_sp_as%kmax-1)
                              ! $ \DP{T}{t} $ . 
                              ! Ѳ. 
                              ! Temperature tendency
    real(DP):: xyz_DQVapDtWork (0:dyn_sp_as%imax-1, 0:dyn_sp_as%jmax-1, 0:dyn_sp_as%kmax-1)
                              ! $ \DP{q}{t} $ . 
                              ! 漾Ѳ. 
                              ! Specific humidity tendency

    real(DP):: wz_DVorDtWork ((dyn_sp_as%nmax+1)**2, 0:dyn_sp_as%kmax-1)
                              ! $ \DD{\zeta}{t} (t) $ . Ѳ (ڥȥ). 
                              ! Vorticity tendency (spectral)
    real(DP):: wz_DDivDtWork ((dyn_sp_as%nmax+1)**2, 0:dyn_sp_as%kmax-1)
                              ! $ \DD{D}{t} (t) $ . ȯѲ (ڥȥ). 
                              ! Divergence tendency (spectral)
    real(DP):: wz_DTempDtWork ((dyn_sp_as%nmax+1)**2, 0:dyn_sp_as%kmax-1)
                              ! $ \DD{T}{t} (t) $ . Ѳ (ڥȥ). 
                              ! Temperature tendency (spectral)
    real(DP):: wz_DQVapDtWork ((dyn_sp_as%nmax+1)**2, 0:dyn_sp_as%kmax-1)
                              ! $ \DD{q}{t} (t) $ . 漾Ѳ (ڥȥ). 
                              ! Specific humidity tendency (spectral)

    !-----------------------------------
    !  ץꥷåˡΤκѿ
    !  Work variables for explicit scheme
    real(DP):: xyz_exWTGPi (0:dyn_sp_as%imax-1, 0:dyn_sp_as%jmax-1, 0:dyn_sp_as%kmax-1)
                              ! $ \underline{W} \Dvect{T} + \Dvect{G} \pi $ .

    real(DP):: xyz_exHDiv (0:dyn_sp_as%imax-1, 0:dyn_sp_as%jmax-1, 0:dyn_sp_as%kmax-1)
                              ! $ \underline{h} \Dvect{D} $ .


    character(TOKEN):: time_integration_scheme
                              ! ʬˡ. 
                              ! ʲˡǽ. 
                              !
                              ! Time integration scheme. 
                              ! Available schemes are as follows.
                              ! 
                              ! * "Semi-implicit"
                              ! * "Explicit"
                              ! 

    !-----------------------------------
    !  ҥȥեؤΥǡ
    !  Configure the settings for history data output
    character(STRING):: name = ''
                              ! ѿ̾. Variable identifier
    real:: time
                              ! . Time
    type(GT_HISTORY), pointer:: gthist =>null()
                              ! gt4_history ⥸塼ѹ¤. 
                              ! Derived type for "gt4_history" module

    !-----------------------------------
    !  ѿ
    !  Work variables
    integer:: stat
    character(STRING):: cause_c
    character(*), parameter:: subname = 'DynSpAsDynamics'
  continue
    call BeginSub(subname)
    stat = DC_NOERR
    cause_c = ''

    !-----------------------------------------------------------------
    !  Υå
    !  Check initialization
    !-----------------------------------------------------------------
    if (.not. dyn_sp_as % initialized) then
      stat = DCPAM_ENOTINIT
      cause_c = 'DYNSPAS83'
      goto 999
    end if


    !-----------------------------------------------------------------
    !  ʬˡ䤤碌
    !  Inquire time integration scheme
    !-----------------------------------------------------------------
    call Inquire( dyn_as = dyn_sp_as % dyn_as, &            ! (in)
      & time_integration_scheme = time_integration_scheme ) ! (out)

    !-----------------------------------------------------------------
    !  Ѳη׻
    !  Calculate tendency terms
    !-----------------------------------------------------------------

    !---------------------------------------------
    !  ɽ̵ζѲη׻
    !  Calculate spatial surface pressure tendency
    !
    call GradPi( dyn_sp = dyn_sp_as % dyn_sp, &  ! (inout)
      & xy_Ps = xy_PsN, &                        ! (in)
      & xy_GradLonPi = xy_GradLonPiN, &          ! (out)
      & xy_GradLatPi = xy_GradLatPiN )           ! (out)

    !---------------------------------------------
    !  ȯ®η׻
    !  Calculate wind velocity from vorticity and divergence
    !
    call VorDiv2UV( dyn_sp = dyn_sp_as % dyn_sp, &  ! (inout)
      & xyz_Vor = xyz_VorN, xyz_Div = xyz_DivN, &   ! (in)
      & xyz_U = xyz_UN, xyz_V = xyz_VN )            ! (out)

    !---------------------------------------------
    !  ʻǤϳعη׻
    !  Calculate non-linear dynamical terms on grid points
    !
    call NonLinearOnGrid( dyn_as = dyn_sp_as % dyn_as, &              ! (inout)
      &        xyz_U = xyz_UN,               xyz_V = xyz_VN, &        ! (in)
      &      xyz_Vor = xyz_VorN,           xyz_Div = xyz_DivN, &      ! (in)
      &     xyz_Temp = xyz_TempN,         xyz_QVap = xyz_QVapN, &     ! (in)
      & xy_GradLonPi = xy_GradLonPiN, xy_GradLatPi = xy_GradLatPiN, & ! (in)
      &     xyz_UAdv = xyz_UAdvN,         xyz_VAdv = xyz_VAdvN, &     ! (out)
      &  xyz_DTempDt = xyz_DTempDtN,   xyz_DQVapDt = xyz_DQVapDtN, &  ! (out)
      &       xyz_KE = xyz_KEN, &                                     ! (out)
      & xyz_TempUAdv = xyz_TempUAdvN, xyz_TempVAdv = xyz_TempVAdvN, & ! (out)
      & xyr_SigmaDot = xyr_SigmaDotN,     xy_DPiDt = xy_DPiDtN, &     ! (out)
      & xyz_QVapUAdv = xyz_QVapUAdvN, xyz_QVapVAdv = xyz_QVapVAdvN )  ! (out)

    !---------------------------------------------
    !  ڥȥѲη׻
    !  Calculate spectral tendency terms
    !
    call Tendency( dyn_sp = dyn_sp_as % dyn_sp, &                     ! (inout)
      &     xyz_UAdv = xyz_UAdvN,         xyz_VAdv = xyz_VAdvN, &     ! (in)
      &       xyz_KE = xyz_KEN, &                                     ! (in)
      & xyz_TempUAdv = xyz_TempUAdvN, xyz_TempVAdv = xyz_TempVAdvN, & ! (in)
      &  xyz_DTempDt = xyz_DTempDtN,      xy_DPiDt = xy_DPiDtN, &     ! (in)
      & xyz_QVapUAdv = xyz_QVapUAdvN, xyz_QVapVAdv = xyz_QVapVAdvN, & ! (in)
      &  xyz_DQVapDt = xyz_DQVapDtN, &                                ! (in)
      &    wz_DVorDt = wz_DVorDtN,       wz_DDivDt = wz_DDivDtN, &    ! (out)
      &   wz_DTempDt = wz_DTempDtN,        w_DPiDt = w_DPiDtN, &      ! (out)
      &   wz_DQVapDt = wz_DQVapDtN )                                  ! (out)

    !---------------------------------------------
    !  ץꥷåˡѤݤη׻
    !  Calculate for explicit scheme
    !
    select case ( LChar( trim( time_integration_scheme ) ) )
    case ('explicit')

      !---------------------------------------------
      !  $ \underline{W} \Dvect{T} + \Dvect{G} \pi $ γʻͤη׻
      !  Calculate $ \underline{W} \Dvect{T} + \Dvect{G} \pi $ on grid
      !
      call WTplusGPiOnGrid( dyn_as = dyn_sp_as % dyn_as, & ! (inout)
        & xyz_Temp = xyz_TempN,   xy_Ps = xy_PsN, &        ! (in)
        & xyz_exWTGPi = xyz_exWTGPi )                      ! (out)

      !---------------------------------------------
      !  $ \underline{h} \Dvect{D} $ γʻͤη׻
      !  Calculate $ \underline{h} \Dvect{D} $ on grid
      !
      call HDivOnGrid( dyn_as = dyn_sp_as % dyn_as, & ! (inout)
        & xyz_Div = xyz_DivN, &                       ! (in)
        & xyz_exHDiv = xyz_exHDiv )                   ! (out)

      !---------------------------------------------
      !  ȯ,  (ڥȥ) λѲν
      !  Modify divergence and temperature tencency (spectral)
      !
      call TendencyExplicit( dyn_sp = dyn_sp_as % dyn_sp, & ! (inout)
        & xyz_exWTGPi = xyz_exWTGPi, &                      ! (in)
        & xyz_exHDiv = xyz_exHDiv, &                        ! (in)
        & wz_DDivDt = wz_DDivDtN, &                         ! (inout)
        & wz_DTempDt = wz_DTempDtN )                        ! (inout)

    end select


    !---------------------------------------------
    !  ʻͤ򥹥ڥȥͤ ( $ t-\Delta t$ )
    !  Exchange grid values to spectral values ( $ t-\Delta t$ )
    !
    call Grid2Spectral( dyn_sp = dyn_sp_as % dyn_sp, & ! (inout)
      & xyz_Data = xyz_VorB, &                         ! (in)
      & wz_Data = wz_VorB )                            ! (out)

    call Grid2Spectral( dyn_sp = dyn_sp_as % dyn_sp, & ! (inout)
      & xyz_Data = xyz_DivB, &                         ! (in)
      & wz_Data = wz_DivB )                            ! (out)

    call Grid2Spectral( dyn_sp = dyn_sp_as % dyn_sp, & ! (inout)
      & xyz_Data = xyz_TempB, &                        ! (in)
      & wz_Data = wz_TempB )                           ! (out)

    call Grid2Spectral( dyn_sp = dyn_sp_as % dyn_sp, & ! (inout)
      & xy_Data = xy_PsB, &                            ! (in)
      & math_func = 'log', &                           ! (in)
      & w_Data = w_PiB )                               ! (out)

    call Grid2Spectral( dyn_sp = dyn_sp_as % dyn_sp, & ! (inout)
      & xyz_Data = xyz_QVapB, &                        ! (in)
      & wz_Data = wz_QVapB )                           ! (out)

    !-----------------------------------------------------------------
    !  ץˤѲʻҥǡ򥹥ڥȥǡѴ
    !  Convert tendency data on grid by external processes into spectral data
    !-----------------------------------------------------------------
    xyz_DVorDtWork  = 0.0_DP
    xyz_DDivDtWork  = 0.0_DP
    xyz_DTempDtWork = 0.0_DP
    xyz_DQVapDtWork = 0.0_DP

    if ( present(xyz_DVorDt ) ) xyz_DVorDtWork  = xyz_DVorDt
    if ( present(xyz_DDivDt ) ) xyz_DDivDtWork  = xyz_DDivDt
    if ( present(xyz_DTempDt) ) xyz_DTempDtWork = xyz_DTempDt
    if ( present(xyz_DQVapDt) ) xyz_DQVapDtWork = xyz_DQVapDt

    !---------------------------------------------
    !  ʻͤ򥹥ڥȥͤ ( $ t-\Delta t$ )
    !  Exchange grid values to spectral values ( $ t-\Delta t$ )
    !
    call Grid2Spectral( dyn_sp = dyn_sp_as % dyn_sp, & ! (inout)
      & xyz_Data = xyz_DVorDtWork, &                   ! (in)
      &  wz_Data = wz_DVorDtWork )                     ! (out)

    call Grid2Spectral( dyn_sp = dyn_sp_as % dyn_sp, & ! (inout)
      & xyz_Data = xyz_DDivDtWork, &                   ! (in)
      &  wz_Data = wz_DDivDtWork )                     ! (out)

    call Grid2Spectral( dyn_sp = dyn_sp_as % dyn_sp, & ! (inout)
      & xyz_Data = xyz_DTempDtWork, &                  ! (in)
      &  wz_Data = wz_DTempDtWork )                    ! (out)

    call Grid2Spectral( dyn_sp = dyn_sp_as % dyn_sp, & ! (inout)
      & xyz_Data = xyz_DQVapDtWork, &                  ! (in)
      &  wz_Data = wz_DQVapDtWork )                    ! (out)

    !-----------------------------------------------------------------
    !  ʬ
    !  Time integration
    !-----------------------------------------------------------------

    call TimeIntegration( dyn_as = dyn_sp_as % dyn_as, &    ! (inout)
      &  wz_DVorDtN = wz_DVorDtN + wz_DVorDtWork, &         ! (in)
      &  wz_DDivDtN = wz_DDivDtN + wz_DDivDtWork, &         ! (in)
      & wz_DTempDtN = wz_DTempDtN + wz_DTempDtWork, &       ! (in)
      & wz_DQVapDtN = wz_DQVapDtN + wz_DQVapDtWork, &       ! (in)
      &    w_DPiDtN = w_DPiDtN, &                           ! (in)
      &     wz_VorB = wz_VorB,     wz_DivB = wz_DivB, &     ! (in)
      &    wz_TempB = wz_TempB,   wz_QVapB = wz_QVapB, &    ! (in)
      &       w_PiB = w_PiB, &                              ! (in)
      &     wz_VorA = wz_VorA,     wz_DivA = wz_DivA, &     ! (out)
      &    wz_TempA = wz_TempA,   wz_QVapA = wz_QVapA, &    ! (out)
      &       w_PiA = w_PiA )                               ! (out)

    !---------------------------------------------
    !  ڥȥͤʻͤ ( $ t+\Delta t$ )
    !  Exchange spectral values to grid values ( $ t+\Delta t$ )
    !
    call Spectral2Grid( dyn_sp = dyn_sp_as % dyn_sp, & ! (inout)
      & wz_Data = wz_VorA, &                           ! (in)
      & xyz_Data = xyz_VorA )                          ! (out)

    call Spectral2Grid( dyn_sp = dyn_sp_as % dyn_sp, & ! (inout)
      & wz_Data = wz_DivA, &                           ! (in)
      & xyz_Data = xyz_DivA )                          ! (out)

    call Spectral2Grid( dyn_sp = dyn_sp_as % dyn_sp, & ! (inout)
      & wz_Data = wz_TempA, &                          ! (in)
      & xyz_Data = xyz_TempA )                         ! (out)

    call Spectral2Grid( dyn_sp = dyn_sp_as % dyn_sp, & ! (inout)
      & w_Data = w_PiA, &                              ! (in)
      & math_func = 'exp', &                           ! (in)
      & xy_Data = xy_PsA )                             ! (out)

    call Spectral2Grid( dyn_sp = dyn_sp_as % dyn_sp, & ! (inout)
      & wz_Data = wz_QVapA, &                          ! (in)
      & xyz_Data = xyz_QVapA )                         ! (out)

    call VorDiv2UV( dyn_sp = dyn_sp_as % dyn_sp, &  ! (inout)
      & xyz_Vor = xyz_VorA, xyz_Div = xyz_DivA, &   ! (in)
      & xyz_U = xyz_UA, xyz_V = xyz_VA )            ! (out)

    !-----------------------------------------------------------------
    !  Ȼˤ
    !  Correction by diffusion
    !-----------------------------------------------------------------

    !---------------------------------------------
    !  ư̿ʿȻˤ뱲ȯλѲ
    !  Vorticity and divergence tendency by 
    !  horizontal diffusion of momentum
    !
    call DiffusionVorDiv( dyn_sp = dyn_sp_as % dyn_sp, &     ! (inout)
      &     wz_Vor = wz_VorA,         wz_Div = wz_DivA, &    ! (in)
      & wz_VorDiff = wz_VorDiffA, wz_DivDiff = wz_DivDiffA ) ! (out)

    !---------------------------------------------
    !  ư̿ʿȻˤ໤Ǯ
    !  Frictional thermal correction by horizontal momentum diffusion
    !
    call DiffusionCorrectTemp( dyn_sp = dyn_sp_as % dyn_sp, & ! (inout)
      & wz_VorDiff = wz_VorDiffA, wz_DivDiff = wz_DivDiffA, & ! (in)
      &      xyz_U = xyz_UA,           xyz_V = xyz_VA, &      ! (in)
      &   xyz_Temp = xyz_TempA )                              ! (inout)

    !----------------------------------------------------------------
    !  ҥȥեؤΥǡ
    !  History data output
    !----------------------------------------------------------------

    !-------------------------
    !  xyr_SigmaDot ν
    !  Output "xyr_SigmaDot"
    name = 'SigmaDot'

    ! ϤΥå. 
    !   * gthist (gt4_history#GT_HISTORY), time (ñټ¿) ꤵ.
    ! Check for output.
    !   * "gthist" (gt4_history#GT_HISTORY), time (real) are configured.
    call output_check ! ֥롼. This is internal subroutine

    ! ϥǡ array Ϥ.
    ! Give output data to argument "array"
    if ( associated( gthist ) ) then
      call HistoryPut( &
        & history = gthist, &                      ! (inout)
        & varname = name, array = xyr_SigmaDotN, & ! (in)
        & time = time, quiet = .false., &          ! (in)
        & err = err )                              ! (out)
    end if

    !-------------------------
    !  xy_DPiDt ν
    !  Output "xy_DPiDt"
    name = 'DPiDt'

    ! ϤΥå. 
    !   * gthist (gt4_history#GT_HISTORY), time (ñټ¿) ꤵ.
    ! Check for output.
    !   * "gthist" (gt4_history#GT_HISTORY), time (real) are configured.
    call output_check ! ֥롼. This is internal subroutine

    ! ϥǡ array Ϥ.
    ! Give output data to argument "array"
    if ( associated( gthist ) ) then
      call HistoryPut( &
        & history = gthist, &                  ! (inout)
        & varname = name, array = xy_DPiDtN, & ! (in)
        & time = time, quiet = .false., &      ! (in)
        & err = err )                          ! (out)
    end if

    !-----------------------------------------------------------------
    !  ι
    !  Update time
    !-----------------------------------------------------------------
    dyn_sp_as % current_time = &
      & dyn_sp_as % current_time + dyn_sp_as % delta_time

    !-----------------------------------------------------------------
    !  λ, 㳰
    !  Termination and Exception handling
    !-----------------------------------------------------------------
999 continue
    call StoreError(stat, subname, err, cause_c)
    call EndSub(subname)

  contains

    subroutine output_check
      !
      ! ѿ *name* Ϥ뤫ɤåޤ. 
      ! Ϥ˴ؤ dyn_sp_as % gthstnml Фޤ. 
      !
      ! ѿ *name* ˴ؤƽϤ褦ꤵƤˤ, 
      ! *gthist* ˽ե gt4_history#GT_HISTORY
      ! ѿ礵ޤ. Ǥʤ, *gthist* ֤ˤޤ. 
      !
      ! ޤ, ߻ *time* ꤷޤ. 
      !
      ! Check whether to output variable *name*. 
      ! Information about output is taken out of "dyn_sp_as % gthstnml". 
      !
      ! When output is done for the variable *name*, *gthist* is 
      ! associated with "gt4_history#GT_HISTORY" variable of
      ! the output file. Otherwise, *gthist* is nullified. 
      !
      ! Moreover, current time is set to *time*. 
      !
      character(TOKEN):: interval_unit
                                ! ҥȥǡνϴֳ֤ñ. 
                                ! Unit for interval of history data output
    continue

      nullify( gthist )
      time = 0.0

      if ( HstNmlInfoOutputValid( dyn_sp_as % gthstnml, name ) ) then
        call HstNmlInfoInquire( &
          & gthstnml = dyn_sp_as % gthstnml, & ! (in)
          & name = name, &                     ! (in)
          & interval_unit = interval_unit )    ! (out)

        time = real( EvalbyUnit( dyn_sp_as % current_time, interval_unit ) )
        if ( present_and_true( historyput_flag ) ) time = 0.0

        call HstNmlInfoAssocGtHist( & 
          & gthstnml = dyn_sp_as % gthstnml, & 
          & name = name, &                     ! (in)
          & history = gthist, err = err )      ! (out)
        if ( present_and_true( err ) ) then
          nullify( gthist )
          return
        end if

        if ( .not. HistoryInitialized( gthist ) ) nullify( gthist )
      end if

    end subroutine output_check

  end subroutine DynSpAsDynamics

  subroutine DynSpAsClose( dyn_sp_as, err )
    !
    ! DYNSPAS83 ѿνλԤޤ.
    ! ʤ, Ϳ줿 *dyn_sp_as*  Create ˤäƽ
    ! Ƥʤ, ץϥ顼ȯޤ.
    !
    ! Deconstructor of "DYNSPAS83".
    ! Note that if *dyn_sp_as* is not initialized by "Create" yet,
    ! error is occurred.
    !
    use dc_trace, only: BeginSub, EndSub
    use dc_types, only: STRING, STDOUT
    use dc_string, only: PutLine, Printf, Split, StrInclude, StoA, JoinChar
    use dcpam_error, only: StoreError, DC_NOERR, DCPAM_ENOTINIT
    use dyn_spectral, only: Close
    use dyn_as83, only: Close
    use gt4_history_nmlinfo, only: HstNmlInfoClose, HstNmlInfoNames, &
      & HstNmlInfoAssocGtHist, HstNmlInfoPutLine
    use gt4_history, only: HistoryClose, HistoryInitialized
    implicit none
    type(DYNSPAS83), intent(inout):: dyn_sp_as
    logical, intent(out), optional:: err
                              ! 㳰ѥե饰.
                              ! ǥեȤǤ, μ³ǥ顼
                              ! , ץ϶λޤ.
                              !  *err* Ϳ,
                              ! ץ϶λ, 
                              ! *err*  .true. ޤ.
                              !
                              ! Exception handling flag. 
                              ! By default, when error occur in 
                              ! this procedure, the program aborts. 
                              ! If this *err* argument is given, 
                              ! .true. is substituted to *err* and 
                              ! the program does not abort. 

    !-----------------------------------
    !  ҥȥեؤΥǡ
    !  Configure the settings for history data output
    character(STRING):: name = ''
                              ! ѿ̾. Variable identifier
    character(STRING):: varnames
                              ! ѿ̾ꥹ. 
                              ! List of variables
    character(TOKEN), pointer:: varnames_array(:) =>null()
                              ! ѿ̾ꥹ. 
                              ! List of variables (array) 
    integer:: i, vnmax
    type(GT_HISTORY), pointer:: gthist =>null()
                              ! gt4_history ⥸塼ѹ¤. 
                              ! Derived type for "gt4_history" module

    !-----------------------------------
    !  ѿ
    !  Work variables
    integer:: stat
    character(STRING):: cause_c
    character(*), parameter:: subname = 'DynSpAsClose'
  continue
    call BeginSub(subname)
    stat = DC_NOERR
    cause_c = ''

    !-----------------------------------------------------------------
    !  Υå
    !  Check initialization
    !-----------------------------------------------------------------
    if (.not. dyn_sp_as % initialized) then
      stat = DCPAM_ENOTINIT
      cause_c = 'DYNSPAS83'
      goto 999
    end if

    !-----------------------------------------------------------------
    !  ڥȥˡ Arakawa and Suarez (1983) ξõ
    !  Clear the settings for spectral method and Arakawa and Suarez (1983)
    !-----------------------------------------------------------------
    call Close(dyn_sp = dyn_sp_as % dyn_sp, & ! (inout)
      & err = err) ! (out)
    call Close(dyn_as = dyn_sp_as % dyn_as, & ! (inout)
      & err = err) ! (out)

    !-----------------------------------------------------------------
    !  ҥȥեؤΥǡϤνλ
    !  Terminate the settings for history data output
    !-----------------------------------------------------------------
    varnames = HstNmlInfoNames( dyn_sp_as % gthstnml )
    call Split( str = varnames, sep = ',', & ! (in)
      & carray = varnames_array )            ! (out)
    vnmax = size( varnames_array )

    do i = 1, vnmax
      name = varnames_array(i)
      if ( trim( name ) == '' ) exit
      nullify( gthist )
      call HstNmlInfoAssocGtHist( & 
        & gthstnml = dyn_sp_as % gthstnml, & ! (in)
        & name = name, &                           ! (in)
        & history = gthist, &                      ! (out)
        & err = err )                              ! (out)
      if ( HistoryInitialized( gthist ) ) then

        call HistoryClose( history = gthist, & ! (inout)
          & err = err )                        ! (out)

      end if
    end do

    !-----------------------------------------------------------------
    !  λ, 㳰
    !  Termination and Exception handling
    !-----------------------------------------------------------------
    dyn_sp_as % initialized = .false.
999 continue
    call StoreError(stat, subname, err, cause_c)
    call EndSub(subname)
  end subroutine DynSpAsClose

  subroutine DynSpAsPutLine( dyn_sp_as, unit, indent, err )
    !
    !  *dyn_sp_as* ꤵƤޤ.
    ! ǥեȤǤϥåɸϤ˽Ϥޤ. 
    ! *unit* ֹꤹ뤳Ȥ, ѹ뤳ȤǽǤ.
    !
    ! Print information of *dyn_sp_as*.
    ! By default messages are output to standard output.
    ! Unit number for output can be changed by *unit* argument.
    !
    use dc_trace, only: BeginSub, EndSub
    use dc_types, only: STRING, STDOUT
    use dcpam_error, only: StoreError, DC_NOERR, DCPAM_ENOTINIT
    use dc_string, only: PutLine, Printf, Split, StrInclude, StoA, JoinChar
    use dyn_spectral, only: PutLine
    use dyn_as83, only: PutLine
    use gt4_history_nmlinfo, only: GTHST_NMLINFO, HstNmlInfoPutLine
    implicit none
    type(DYNSPAS83), intent(inout):: dyn_sp_as
    integer, intent(in), optional:: unit
                              ! ֹ.
                              ! ǥեȤνɸ.
                              !
                              ! Unit number for output.
                              ! Default value is standard output.
    character(*), intent(in), optional:: indent
                              ! ɽåλ.
                              !
                              ! Indent of displayed messages.
    logical, intent(out), optional:: err
                              ! 㳰ѥե饰.
                              ! ǥեȤǤ, μ³ǥ顼
                              ! , ץ϶λޤ.
                              !  *err* Ϳ,
                              ! ץ϶λ, 
                              ! *err*  .true. ޤ.
                              !
                              ! Exception handling flag. 
                              ! By default, when error occur in 
                              ! this procedure, the program aborts. 
                              ! If this *err* argument is given, 
                              ! .true. is substituted to *err* and 
                              ! the program does not abort. 


    !-----------------------------------
    !  ѿ
    !  Work variables
    integer:: stat
    character(STRING):: cause_c
    integer:: out_unit
    integer:: indent_len
    character(STRING):: indent_str
    character(*), parameter:: subname = 'DynSpAsPutLine'
  continue
    call BeginSub(subname)
    stat = DC_NOERR
    cause_c = ''

    !-----------------------------------------------------------------
    !  Υå
    !  Check initialization
    !-----------------------------------------------------------------
    if (present(unit)) then
      out_unit = unit
    else
      out_unit = STDOUT
    end if

    indent_len = 0
    indent_str = ''
    if (present(indent)) then
      if (len(indent) /= 0) then
        indent_len = len(indent)
        indent_str(1:indent_len) = indent
      end if
    end if

    !-----------------------------------------------------------------
    !  "DYNSPAS83" ΰ
    !  Print the settings for "DYNSPAS83"
    !-----------------------------------------------------------------
    if (dyn_sp_as % initialized) then
      call Printf(out_unit, &
        & indent_str(1:indent_len) // &
        & '#<DYNSPAS83:: @initialized=%y', &
        & l=(/dyn_sp_as % initialized/))

      call PutLine(dyn_sp_as % dyn_sp, out_unit, &
        & indent_str(1:indent_len) // &
        & ' ', err)
      call PutLine(dyn_sp_as % dyn_as, out_unit, &
        & indent_str(1:indent_len) // &
        & ' ', err)

      call HstNmlInfoPutLine( &
        & gthstnml = dyn_sp_as % gthstnml, &          ! (in)
        & unit = out_unit, &                          ! (in)
        & indent = indent_str(1:indent_len) // '  ' ) ! (in)

      call Printf( out_unit, &
        & indent_str(1:indent_len) // &
        & '>' )
    else
      call Printf(out_unit, &
        & indent_str(1:indent_len) // &
        & '#<DYNSPAS83:: @initialized=%y>', &
        & l=(/dyn_sp_as % initialized/))
    end if

    !-----------------------------------------------------------------
    !  λ, 㳰
    !  Termination and Exception handling
    !-----------------------------------------------------------------
999 continue
    call StoreError(stat, subname, err, cause_c)
    call EndSub(subname)
  end subroutine DynSpAsPutLine

  logical function DynSpAsInitialized( dyn_sp_as ) result(result)
    !
    ! *dyn_sp_as* ꤵƤˤ .true. ,
    ! ꤵƤʤˤ .false. ֤ޤ.
    !
    ! If *dyn_sp_as* is initialized, .true. is returned.
    ! If *dyn_sp_as* is not initialized, .false. is returned.
    !
    implicit none
    type(DYNSPAS83), intent(in):: dyn_sp_as
  continue
    result = dyn_sp_as % initialized
  end function DynSpAsInitialized

  subroutine DynSpAsSetTime( dyn_sp_as, &
    & current_time_value, current_time_unit, &
    & err )
    !
    ! *dyn_sp_as* ФƻԤޤ. 
    !
    ! ҥȥǡϤƤˤ, ҥȥǡ
    ! ϻꤷޤ. ٤Ǥ⤳Υ֥롼Ƥˤ, 
    ! ʸΥҥȥǡˤΥ֥롼ƤӽФ, 
    ! ԤäƤ. 
    ! ޤ, ǡϤ륵֥롼ФƤ 
    ! ץʥ historyput_flag  .true. ͿƤ. 
    !
    ! ʤ, Ϳ줿 *dyn_sp_as*  Create ˤäƽ
    ! Ƥʤ, ץϥ顼ȯޤ.
    !
    ! Set time to *dyn_sp_as*.
    !
    ! When history data are output, the output time of history data 
    ! are specified. 
    ! Once this subroutine is called, the time of history data must be
    ! specified by this routine before history data output
    ! In additional, give ".true." to an optional argument 
    ! "historyput_flag" of a data output subroutine. 
    !
    ! If *dyn_sp_as* is not initialized by "Create" yet,
    ! error is occurred.
    !
    use dc_trace, only: BeginSub, EndSub
    use dc_string, only: PutLine, Printf, Split, StrInclude, StoA, JoinChar
    use dc_types, only: DP, STRING, TOKEN, STDOUT
    use dc_date, only: DCDiffTimeCreate, EvalbyUnit
    use dc_error, only: StoreError, DC_NOERR, DC_ENOTINIT
    use gt4_history_nmlinfo, only: HstNmlInfoAdd, HstNmlInfoInquire, &
      & HstNmlInfoNames, HstNmlInfoAssocGtHist, &
      & HstNmlInfoOutputStepDisable, HstNmlInfoPutLine
    use gt4_history, only: HistorySetTime, HistoryInitialized
    implicit none
    type(DYNSPAS83), intent(inout):: dyn_sp_as
    real(DP), intent(in):: current_time_value
                              ! ߻ο. Numerical value of current time
    character(*), intent(in):: current_time_unit
                              ! ߻ñ. Unit of current time
    logical, intent(out), optional:: err
                              ! 㳰ѥե饰.
                              ! ǥեȤǤ, μ³ǥ顼
                              ! , ץ϶λޤ.
                              !  *err* Ϳ,
                              ! ץ϶λ, 
                              ! *err*  .true. ޤ.
                              !
                              ! Exception handling flag. 
                              ! By default, when error occur in 
                              ! this procedure, the program aborts. 
                              ! If this *err* argument is given, 
                              ! .true. is substituted to *err* and 
                              ! the program does not abort. 

    !-----------------------------------
    !  ҥȥեؤΥǡ
    !  Configure the settings for history data output
    character(STRING):: name = ''
                              ! ѿ̾. Variable identifier
    character(TOKEN):: interval_unit
                              ! ҥȥǡνϴֳ֤ñ. 
                              ! Unit for interval of history data output
    character(STRING):: varnames
                              ! ѿ̾ꥹ. 
                              ! List of variables
    character(TOKEN), pointer:: varnames_array(:) =>null()
                              ! ѿ̾ꥹ. 
                              ! List of variables (array) 
    integer:: i, vnmax
    type(GT_HISTORY), pointer:: gthist =>null()
                              ! gt4_history ⥸塼ѹ¤. 
                              ! Derived type for "gt4_history" module

    !-----------------------------------
    !  ѿ
    !  Work variables
    integer:: stat
    character(STRING):: cause_c
    character(*), parameter:: subname = 'DynSpAsSetTime'
  continue
    call BeginSub( subname )
    stat = DC_NOERR
    cause_c = ''

    !-----------------------------------------------------------------
    !  Υå
    !  Check initialization
    !-----------------------------------------------------------------
    if ( .not. dyn_sp_as % initialized ) then
      stat = DC_ENOTINIT
      cause_c = 'DYNSPAS83'
      goto 999
    end if

    !-----------------------------------------------------------------
    !  
    !  Configure time
    !-----------------------------------------------------------------
    call DCDiffTimeCreate( &
      & diff = dyn_sp_as % current_time, & ! (out)
      & value = current_time_value, &    ! (in)
      & unit = current_time_unit )       ! (in)

    !-----------------------------------------------------------------
    !  ҥȥեؤΥǡλ
    !  Configure the time of history data
    !-----------------------------------------------------------------
    varnames = HstNmlInfoNames( dyn_sp_as % gthstnml )
    call Split( str = varnames, sep = ',', & ! (in)
      & carray = varnames_array )            ! (out)
    vnmax = size( varnames_array )

    do i = 1, vnmax
      name = varnames_array(i)
      if ( trim( name ) == '' ) exit

      call HstNmlInfoOutputStepDisable( & 
        & gthstnml = dyn_sp_as % gthstnml, & ! (inout)
        & name = name, &                           ! (in)
        & err = err )                              ! (out)

      nullify( gthist )
      call HstNmlInfoAssocGtHist( & 
        & gthstnml = dyn_sp_as % gthstnml, & ! (in)
        & name = name, &                           ! (in)
        & history = gthist, &                      ! (out)
        & err = err )                              ! (out)

      if ( HistoryInitialized( gthist ) ) then
        call HstNmlInfoInquire( &
          & gthstnml = dyn_sp_as % gthstnml, & ! (in)
          & name = name, &                           ! (in)
          & interval_unit = interval_unit )          ! (out)

        call HistorySetTime( &
          & history = gthist, &                     ! (inout)
          & time = &
          &   real( EvalbyUnit( dyn_sp_as % current_time, &
          &                     interval_unit) ) ) ! (in)
      end if

    end do

    !-----------------------------------------------------------------
    !  λ, 㳰
    !  Termination and Exception handling
    !-----------------------------------------------------------------
999 continue
    call StoreError( stat, subname, err, cause_c )
    call EndSub( subname )
  end subroutine DynSpAsSetTime

  subroutine DynSpAsEqualAxes( dyn_sp_as, &
    & x_Lon, y_Lat, z_Sigma, r_Sigma, &
    & err )
    !
    ! Ϳ줿ɸǡ  *dyn_sp_as* 
    ! ݻɸǡȤǧޤ.
    ! ۤʤˤϥ顼ȯޤ.
    !
    ! ʤ, Ϳ줿 *dyn_sp_as*  Create ˤäƽ
    ! Ƥʤ, ץϥ顼ȯޤ.
    !
    ! Confirm equality between given axes data and 
    ! axes data stored in *dyn_sp_as*. 
    ! If the equality is not confirmed, error is occurred.
    !
    ! If *dyn_sp_as* is not initialized by "Create" yet,
    ! error is occurred.
    !
    use dc_trace, only: BeginSub, EndSub
    use dc_types, only: STRING, STDOUT, DP
    use dc_string, only: PutLine, Printf, Split, StrInclude, StoA, JoinChar
    use dcpam_error, only: StoreError, DC_NOERR, DCPAM_ENOTINIT, DCPAM_EAXISMISMATCH
    use dyn_spectral, only: EqualAxes
    use dyn_as83, only: EqualAxes
    implicit none
    type(DYNSPAS83), intent(inout):: dyn_sp_as
    real(DP), intent(in), optional:: x_Lon (0:dyn_sp_as%imax-1)
                              ! . Longitude
    real(DP), intent(in), optional:: y_Lat (0:dyn_sp_as%jmax-1)
                              ! . Latitude
    real(DP), intent(in), optional:: z_Sigma (0:dyn_sp_as%kmax-1)
                              ! $ \sigma $ ٥ (). 
                              ! Full $ \sigma $ level
    real(DP), intent(in), optional:: r_Sigma (0:dyn_sp_as%kmax)
                              ! $ \sigma $ ٥ (Ⱦ). 
                              ! Half $ \sigma $ level
    logical, intent(out), optional:: err
                              ! 㳰ѥե饰.
                              ! ǥեȤǤ, μ³ǥ顼
                              ! , ץ϶λޤ.
                              !  *err* Ϳ,
                              ! ץ϶λ, 
                              ! *err*  .true. ޤ.
                              !
                              ! Exception handling flag. 
                              ! By default, when error occur in 
                              ! this procedure, the program aborts. 
                              ! If this *err* argument is given, 
                              ! .true. is substituted to *err* and 
                              ! the program does not abort. 
    integer:: stat
    character(STRING):: cause_c
    character(*), parameter:: subname = 'DynSpAsEqualAxes'
  continue
    call BeginSub(subname)
    stat = DC_NOERR
    cause_c = ''

    !-----------------------------------------------------------------
    !  Υå
    !  Check initialization
    !-----------------------------------------------------------------
    if (.not. dyn_sp_as % initialized) then
      stat = DCPAM_ENOTINIT
      cause_c = 'DYNSPAS83'
      goto 999
    end if

    !-----------------------------------------------------------------
    !  ǡ
    !  Compare axes data
    !-----------------------------------------------------------------
    call EqualAxes( dyn_sp = dyn_sp_as % dyn_sp, & ! (inout)
      & x_Lon = x_Lon, y_Lat = y_Lat, &            ! (in)
      & err = err )                                ! (out)

    call EqualAxes( dyn_as = dyn_sp_as % dyn_as, & ! (inout)
      & z_Sigma = z_Sigma, r_Sigma = r_Sigma, &    ! (in)
      & err = err )                                ! (out)

    !-----------------------------------------------------------------
    !  λ, 㳰
    !  Termination and Exception handling
    !-----------------------------------------------------------------
999 continue
    call StoreError(stat, subname, err, cause_c)
    call EndSub(subname)
  end subroutine DynSpAsEqualAxes

  subroutine DynSpAsGetAxes( dyn_sp_as, &
    & x_Lon, y_Lat, z_Sigma, r_Sigma, z_DelSigma, &
    & err )
    !
    ! *dyn_sp_as* ˳ǼƤɸ֤ͤޤ.
    !
    ! ʤ, Ϳ줿 *dyn_sp_as*  Create ˤäƽ
    ! Ƥʤ, ץϥ顼ȯޤ.
    !
    ! Return data of axes stored in *dyn_sp_as*.
    !
    ! If *dyn_sp_as* is not initialized by "Create" yet,
    ! error is occurred.
    !
    use dc_trace, only: BeginSub, EndSub
    use dc_types, only: STRING, STDOUT, DP
    use dc_string, only: PutLine, Printf, Split, StrInclude, StoA, JoinChar
    use dcpam_error, only: StoreError, DC_NOERR, DCPAM_ENOTINIT
    use dyn_spectral, only: GetAxes
    use dyn_as83, only: GetAxes
    implicit none
    type(DYNSPAS83), intent(inout):: dyn_sp_as
    real(DP), intent(out):: x_Lon (0:dyn_sp_as%imax-1)
                              ! . Longitude
    real(DP), intent(out):: y_Lat (0:dyn_sp_as%jmax-1)
                              ! . Latitude
    real(DP), intent(out):: z_Sigma (0:dyn_sp_as%kmax-1)
                              ! $ \sigma $ ٥ (). 
                              ! Full $ \sigma $ level
    real(DP), intent(out):: r_Sigma (0:dyn_sp_as%kmax)
                              ! $ \sigma $ ٥ (Ⱦ). 
                              ! Half $ \sigma $ level
    real(DP), intent(out), optional:: z_DelSigma (0:dyn_sp_as%kmax-1)
                              ! $ \Delta \sigma $ (). 
                              ! $ \Delta \sigma $ (Full)

    logical, intent(out), optional:: err
                              ! 㳰ѥե饰.
                              ! ǥեȤǤ, μ³ǥ顼
                              ! , ץ϶λޤ.
                              !  *err* Ϳ,
                              ! ץ϶λ, 
                              ! *err*  .true. ޤ.
                              !
                              ! Exception handling flag. 
                              ! By default, when error occur in 
                              ! this procedure, the program aborts. 
                              ! If this *err* argument is given, 
                              ! .true. is substituted to *err* and 
                              ! the program does not abort. 
    integer:: stat
    character(STRING):: cause_c
    character(*), parameter:: subname = 'DynSpAsSample'
  continue
    call BeginSub(subname)
    stat = DC_NOERR
    cause_c = ''

    !-----------------------------------------------------------------
    !  Υå
    !  Check initialization
    !-----------------------------------------------------------------
    if (.not. dyn_sp_as % initialized) then
      stat = DCPAM_ENOTINIT
      cause_c = 'DYNSPAS83'
      goto 999
    end if

    !-----------------------------------------------------------------
    !  ٷٺɸͤμ
    !  Get data of latitude and longitude
    !-----------------------------------------------------------------
    call GetAxes( dyn_sp = dyn_sp_as % dyn_sp, &  ! (inout)
      & x_Lon = x_Lon, y_Lat = y_Lat ) ! (out)

    !-----------------------------------------------------------------
    !  ľ٥ɸͤμ
    !  Get data of vertical level
    !-----------------------------------------------------------------
    call GetAxes( dyn_as = dyn_sp_as % dyn_as, &  ! (inout)
      & z_Sigma = z_Sigma, r_Sigma = r_Sigma, & ! (out)
      & z_DelSigma = z_DelSigma ) ! (out)

    !-----------------------------------------------------------------
    !  λ, 㳰
    !  Termination and Exception handling
    !-----------------------------------------------------------------
999 continue
    call StoreError(stat, subname, err, cause_c)
    call EndSub(subname)
  end subroutine DynSpAsGetAxes


  subroutine DynSpAsVorDiv2UV( dyn_sp_as, &
    & xyz_Vor, xyz_Div, &
    & xyz_U, xyz_V, &
    & err )
    !
    ! Ϳ줿٤ȯ®®׻ޤ.
    !
    ! ʤ, Ϳ줿 *dyn_sp_as*  Create ˤäƽ
    ! Ƥʤ, ץϥ顼ȯޤ.
    !
    ! Calculate zonal and meridional wind from given 
    ! vorticity and divergence.
    !
    ! If *dyn_sp_as* is not initialized by "Create" yet,
    ! error is occurred.
    !
    use dc_trace, only: BeginSub, EndSub
    use dc_types, only: STRING, STDOUT, DP
    use dc_string, only: PutLine, Printf, Split, StrInclude, StoA, JoinChar
    use dcpam_error, only: StoreError, DC_NOERR, DCPAM_ENOTINIT
    use dyn_spectral, only: VorDiv2UV
    implicit none
    type(DYNSPAS83), intent(inout):: dyn_sp_as
    real(DP), intent(in):: xyz_Vor (0:dyn_sp_as%imax-1, 0:dyn_sp_as%jmax-1, 0:dyn_sp_as%kmax-1)
                              !  $ \zeta $
    real(DP), intent(in):: xyz_Div (0:dyn_sp_as%imax-1, 0:dyn_sp_as%jmax-1, 0:dyn_sp_as%kmax-1)
                              ! ȯ $ D $
    real(DP), intent(out):: xyz_U (0:dyn_sp_as%imax-1, 0:dyn_sp_as%jmax-1, 0:dyn_sp_as%kmax-1)
                              ! ® $ U $
    real(DP), intent(out):: xyz_V (0:dyn_sp_as%imax-1, 0:dyn_sp_as%jmax-1, 0:dyn_sp_as%kmax-1)
                              ! ® $ V $
    logical, intent(out), optional:: err
                              ! 㳰ѥե饰.
                              ! ǥեȤǤ, μ³ǥ顼
                              ! , ץ϶λޤ.
                              !  *err* Ϳ,
                              ! ץ϶λ, 
                              ! *err*  .true. ޤ.
                              !
                              ! Exception handling flag. 
                              ! By default, when error occur in 
                              ! this procedure, the program aborts. 
                              ! If this *err* argument is given, 
                              ! .true. is substituted to *err* and 
                              ! the program does not abort. 
    integer:: stat
    character(STRING):: cause_c
    character(*), parameter:: subname = 'DynSpAsVorDiv2UV'
  continue
    call BeginSub(subname)
    stat = DC_NOERR
    cause_c = ''

    !-----------------------------------------------------------------
    !  Υå
    !  Check initialization
    !-----------------------------------------------------------------
    if (.not. dyn_sp_as % initialized) then
      stat = DCPAM_ENOTINIT
      cause_c = 'DYNSPAS83'
      goto 999
    end if

    !-----------------------------------------------------------------
    !  ȯ®η׻
    !  Calculate wind velocity from vorticity and divergence
    !-----------------------------------------------------------------
    call VorDiv2UV( dyn_sp = dyn_sp_as % dyn_sp, & ! (inout)
      & xyz_Vor = xyz_Vor, xyz_Div = xyz_Div, &    ! (in)
      & xyz_U = xyz_U, xyz_V = xyz_V )             ! (out)

    !-----------------------------------------------------------------
    !  λ, 㳰
    !  Termination and Exception handling
    !-----------------------------------------------------------------
999 continue
    call StoreError(stat, subname, err, cause_c)
    call EndSub(subname)
  end subroutine DynSpAsVorDiv2UV

  subroutine DynSpAsUV2VorDiv( dyn_sp_as, &
    & xyz_Vor, xyz_Div, &
    & xyz_U, xyz_V, &
    & err )
    !
    ! Ϳ줿®®鱲٤ȯ׻ޤ.
    !
    ! ʤ, Ϳ줿 *dyn_sp_as*  Create ˤäƽ
    ! Ƥʤ, ץϥ顼ȯޤ.
    !
    ! Calculate vorticity and divergence
    ! from given zonal and meridional wind.
    !
    ! If *dyn_sp_as* is not initialized by "Create" yet,
    ! error is occurred.
    !
    use dc_trace, only: BeginSub, EndSub
    use dc_types, only: STRING, STDOUT, DP
    use dc_string, only: PutLine, Printf, Split, StrInclude, StoA, JoinChar
    use dcpam_error, only: StoreError, DC_NOERR, DCPAM_ENOTINIT
    use dyn_spectral, only: UV2VorDiv
    implicit none
    type(DYNSPAS83), intent(inout):: dyn_sp_as
    real(DP), intent(in):: xyz_U (0:dyn_sp_as%imax-1, 0:dyn_sp_as%jmax-1, 0:dyn_sp_as%kmax-1)
                              ! ® $ U $
    real(DP), intent(in):: xyz_V (0:dyn_sp_as%imax-1, 0:dyn_sp_as%jmax-1, 0:dyn_sp_as%kmax-1)
                              ! ® $ V $
    real(DP), intent(out):: xyz_Vor (0:dyn_sp_as%imax-1, 0:dyn_sp_as%jmax-1, 0:dyn_sp_as%kmax-1)
                              !  $ \zeta $
    real(DP), intent(out):: xyz_Div (0:dyn_sp_as%imax-1, 0:dyn_sp_as%jmax-1, 0:dyn_sp_as%kmax-1)
                              ! ȯ $ D $
    logical, intent(out), optional:: err
                              ! 㳰ѥե饰.
                              ! ǥեȤǤ, μ³ǥ顼
                              ! , ץ϶λޤ.
                              !  *err* Ϳ,
                              ! ץ϶λ, 
                              ! *err*  .true. ޤ.
                              !
                              ! Exception handling flag. 
                              ! By default, when error occur in 
                              ! this procedure, the program aborts. 
                              ! If this *err* argument is given, 
                              ! .true. is substituted to *err* and 
                              ! the program does not abort. 
    integer:: stat
    character(STRING):: cause_c
    character(*), parameter:: subname = 'DynSpAsUV2VorDiv'
  continue
    call BeginSub(subname)
    stat = DC_NOERR
    cause_c = ''

    !-----------------------------------------------------------------
    !  Υå
    !  Check initialization
    !-----------------------------------------------------------------
    if (.not. dyn_sp_as % initialized) then
      stat = DCPAM_ENOTINIT
      cause_c = 'DYNSPAS83'
      goto 999
    end if

    !-----------------------------------------------------------------
    !  ®鱲ȯη׻
    !  Calculate vorticity and divergence from wind velocity
    !-----------------------------------------------------------------
    call UV2VorDiv( dyn_sp = dyn_sp_as % dyn_sp, & ! (inout)
      & xyz_U = xyz_U, xyz_V = xyz_V, &            ! (in)
      & xyz_Vor = xyz_Vor, xyz_Div = xyz_Div )     ! (out)

    !-----------------------------------------------------------------
    !  λ, 㳰
    !  Termination and Exception handling
    !-----------------------------------------------------------------
999 continue
    call StoreError(stat, subname, err, cause_c)
    call EndSub(subname)
  end subroutine DynSpAsUV2VorDiv


!!$  subroutine DynSpAsSample( dyn_sp_as, err )
!!$    !--
!!$    ! DynSpAsSample 򵭽ҤƤ.
!!$    !++
!!$    ! ʤ, Ϳ줿 *dyn_sp_as*  Create ˤäƽ
!!$    ! Ƥʤ, ץϥ顼ȯޤ.
!!$    !--
!!$    ! Describe brief of DynSpAsSample
!!$    !++
!!$    ! If *dyn_sp_as* is not initialized by "Create" yet,
!!$    ! error is occurred.
!!$    !
!!$    use dc_trace, only: BeginSub, EndSub
!!$    use dc_types, only: STRING, STDOUT
!!$    use dc_string, only: PutLine, Printf, Split, StrInclude, StoA, JoinChar
!!$    use dcpam_error, only: StoreError, DC_NOERR, DCPAM_ENOTINIT
!!$    implicit none
!!$    type(DYNSPAS83), intent(inout):: dyn_sp_as
!!$    logical, intent(out), optional:: err
!!$                              ! 㳰ѥե饰.
!!$                              ! ǥեȤǤ, μ³ǥ顼
!!$                              ! , ץ϶λޤ.
!!$                              !  *err* Ϳ,
!!$                              ! ץ϶λ, 
!!$                              ! *err*  .true. ޤ.
!!$                              !
!!$                              ! Exception handling flag. 
!!$                              ! By default, when error occur in 
!!$                              ! this procedure, the program aborts. 
!!$                              ! If this *err* argument is given, 
!!$                              ! .true. is substituted to *err* and 
!!$                              ! the program does not abort. 
!!$    integer:: stat
!!$    character(STRING):: cause_c
!!$    character(*), parameter:: subname = 'DynSpAsSample'
!!$  continue
!!$    call BeginSub(subname)
!!$    stat = DC_NOERR
!!$    cause_c = ''
!!$
!!$    !-----------------------------------------------------------------
!!$    !  Υå
!!$    !  Check initialization
!!$    !-----------------------------------------------------------------
!!$    if (.not. dyn_sp_as % initialized) then
!!$      stat = DCPAM_ENOTINIT
!!$      cause_c = 'DYNSPAS83'
!!$      goto 999
!!$    end if
!!$
!!$
!!$
!!$    !-----------------------------------------------------------------
!!$    !  λ, 㳰
!!$    !  Termination and Exception handling
!!$    !-----------------------------------------------------------------
!!$999 continue
!!$    call StoreError(stat, subname, err, cause_c)
!!$    call EndSub(subname)
!!$  end subroutine DynSpAsSample


  subroutine DynSpAsNmlRead( nmlfile, &
!!$    & CoefAlpha, &
!!$    & key00_, &
    & gthstnml, &
    & err )
    !
    ! NAMELIST ե *nmlfile* ͤϤ뤿
    ! ֥롼Ǥ. DynSpAsCreate ǸƤӽФ뤳Ȥ
    ! ꤷƤޤ.
    !
    ! ͤ NAMELIST եǻꤵƤʤˤ,
    ! Ϥ줿ͤΤޤ֤ޤ.
    !
    ! ʤ, *nmlfile* ˶ʸͿ줿, ޤ
    ! Ϳ줿 *nmlfile* ɤ߹ळȤǤʤ, 
    ! ץϥ顼ȯޤ.
    !
    ! This is an internal subroutine to input values from 
    ! NAMELIST file *nmlfile*. This subroutine is expected to be
    ! called by "DynSpAsCreate".
    !
    ! A value not specified in NAMELIST file is returned
    ! without change.
    !
    ! If *nmlfile* is empty, or *nmlfile* can not be read, 
    ! error is occurred.
    !
    use dc_trace, only: BeginSub, EndSub
    use dc_string, only: PutLine, Printf, Split, StrInclude, StoA, JoinChar
    use dc_types, only: DP, STRING, TOKEN, STDOUT
    use dc_iounit, only: FileOpen
    use dc_message, only: MessageNotify
    use dc_present, only: present_and_true
    use dc_date, only: DCDiffTimeCreate
    use dc_error, only: StoreError, DC_NOERR, DC_ENOFILEREAD, DC_ENOASSOC
    use gt4_history_nmlinfo, only: GTHST_NMLINFO, HstNmlInfoAdd, &
      & HstNmlInfoInquire, HstNmlInfoInitialized, HstNmlInfoPutLine
    implicit none
    character(*), intent(in):: nmlfile
                              ! NAMELIST ե̾. 
                              ! NAMELIST file name

!!$    real(DP), intent(inout):: CoefAlpha
!!$                              ! $ \alpha $ . . Coefficient
!!$
!!$    character(*), intent(inout):: key00_
!!$    character(TOKEN):: key00
!!$                              ! . Keyword
!!$

    type(GTHST_NMLINFO), intent(inout):: gthstnml
                              ! NAMELIST#dcmodel_sample_code_history_nml 
                              ! ꤵ̤ΥǡϾ. 
                              ! 
                              ! ǥեͤʤɤ
                              ! ԤäͿ뤳. 
                              ! 
                              ! Individual data output information from 
                              ! "NAMELIST#dcmodel_sample_code_history_nml". 
                              ! 
                              ! Before this argument is given to 
                              ! this procedure, initialize and 
                              ! configure the defaut settings. 
    logical, intent(out), optional:: err
                              ! 㳰ѥե饰.
                              ! ǥեȤǤ, μ³ǥ顼
                              ! , ץ϶λޤ.
                              !  *err* Ϳ,
                              ! ץ϶λ, 
                              ! *err*  .true. ޤ.
                              !
                              ! Exception handling flag. 
                              ! By default, when error occur in 
                              ! this procedure, the program aborts. 
                              ! If this *err* argument is given, 
                              ! .true. is substituted to *err* and 
                              ! the program does not abort. 

!!$    namelist /dyn_spectral_as83_nml/ &
!!$      & CoefAlpha, key00
                              ! dyn_spectral_as83 ⥸塼
                              ! NAMELIST ѿ̾.
                              !
                              ! dyn_spectral_as83#DynSpAsCreate Ѥݤ, 
                              ! ץʥ *nmlfile*  NAMELIST 
                              ! ե̾ꤹ뤳Ȥ, Υե뤫
                              !  NAMELIST ѿɤ߹ߤޤ.
                              !
                              ! NAMELIST group name for
                              ! "dyn_spectral_as83" module.
                              ! 
                              ! If a NAMELIST filename is specified to 
                              ! an optional argument *nmlfile* 
                              ! when "dyn_spectral_as83#DynSpAsCreate" is used, 
                              ! this NAMELIST group is loaded from 
                              ! the file.

    character(STRING):: name
                              ! ѿ̾. 
                              ! ξˤ, ¾ͤ
                              ! dcmodel_sample_code ⥸塼ˤ
                              ! ϤǡƤ
                              ! ǥեͤȤʤޤ. 
                              ! 
                              ! "Data1,Data2" Τ褦˥ޤǶڤäʣ
                              ! ѿꤹ뤳ȤǽǤ. 
                              ! 
                              ! Variable identifier. 
                              ! If blank is given, other values are 
                              ! used as default values of output data 
                              ! in "dcmodel_sample_code". 
                              ! 
                              ! Multiple variables can be specified 
                              ! as "Data1,Data2" too. Delimiter is comma. 
    character(STRING):: file
                              ! ϥե̾. 
                              ! ϥǥեͤȤƤϻѤޤ. 
                              ! *name* ͤꤵƤΤͭǤ. 
                              ! 
                              ! Output file name. 
                              ! This is not used as default value. 
                              ! This value is valid only when *name* is 
                              ! specified. 

    real:: interval_value
                              ! ҥȥǡνϴֳ֤ο. 
                              ! ͤͿ, Ϥ޻ߤޤ. 
                              ! Numerical value for interval of history data output
                              ! Negative values suppresses output.
    character(TOKEN):: interval_unit
                              ! ҥȥǡνϴֳ֤ñ. 
                              ! Unit for interval of history data output
    character(TOKEN):: precision
                              ! ҥȥǡ. 
                              ! Precision of history data
    logical:: average
                              ! ϥǡʿѲե饰. 
                              ! Flag for average of output data
    character(STRING):: fileprefix
                              ! ҥȥǡΥե̾Ƭ. 
                              ! Prefixes of history data filenames

    namelist /dyn_spectral_as83_history_nml/ &
      & name, &
      & file, &
      & interval_value, &
      & interval_unit, &
      & precision, &
      & fileprefix, &
      & average
                              ! dyn_spectral_as83 ⥸塼Υҥȥǡ
                              ! NAMELIST ѿ̾.
                              !
                              ! dyn_spectral_as83#DynSpAsCreate Ѥݤ, 
                              ! ץʥ *nmlfile*  NAMELIST 
                              ! ե̾ꤹ뤳Ȥ, Υե뤫
                              !  NAMELIST ѿɤ߹ߤޤ.
                              !
                              ! NAMELIST group name for 
                              ! history data of "dyn_spectral_as83" module.
                              ! 
                              ! If a NAMELIST filename is specified to 
                              ! an optional argument *nmlfile* 
                              ! when "dyn_spectral_as83#DynSpAsCreate" is used, 
                              ! this NAMELIST group is loaded from 
                              ! the file.


    !-----------------------------------
    !  ѿ
    !  Work variables
    integer:: stat
    character(STRING):: cause_c
    integer:: unit_nml        ! NAMELIST ե륪ץֹ. 
                              ! Unit number for NAMELIST file open
    integer:: iostat_nml      ! NAMELIST ɤ߹߻ IOSTAT. 
                              ! IOSTAT of NAMELIST read
    character(TOKEN):: pos_nml
                              ! NAMELIST ɤ߹߻Υե. 
                              ! File position of NAMELIST read
    character(*), parameter:: subname = 'DynSpAsNmlRead'
  continue
    call BeginSub( subname )
    stat = DC_NOERR
    cause_c = ''

    !-----------------------------------------------------------------
    !  Υå
    !  Check initialization
    !-----------------------------------------------------------------

    !-----------------------------------------------------------------
    !  ʸ NAMELIST ѿ
    !  Substitute character arguments to NAMELIST group
    !-----------------------------------------------------------------
!!$    key00       = key00_

    !----------------------------------------------------------------
    !  NAMELIST եΥץ
    !  Open NAMELIST file
    !----------------------------------------------------------------
    call FileOpen( unit = unit_nml, & ! (out)
      & file = nmlfile, mode = 'r', & ! (in)
      & err = err )                   ! (out)
    if ( present_and_true(err) ) then
      stat = DC_ENOFILEREAD
      cause_c = nmlfile
      goto 999
    end if


    !-----------------------------------------------------------------
    !  NAMELIST ѿμ
    !  Get NAMELIST group
    !-----------------------------------------------------------------
    !-------------------------
    !  ʤɤμ
    !  Get coefficients etc.
!!$    rewind( unit_nml )
!!$    read( unit = unit_nml, & ! (in)
!!$      & nml = dyn_spectral_as83_nml, iostat = iostat_nml ) ! (out)
!!$    if ( iostat_nml == 0 ) then
!!$      call MessageNotify( 'M', subname, &
!!$        & 'NAMELIST group "%c" is loaded from "%c".', &
!!$        & c1 = 'dyn_spectral_as83_nml', c2 = trim(nmlfile) )
!!$      write(STDOUT, nml = dyn_spectral_as83_nml)
!!$    else
!!$      call MessageNotify( 'W', subname, &
!!$        & 'NAMELIST group "%c" is not found in "%c" (iostat=%d).', &
!!$        & c1 = 'dyn_spectral_as83_nml', c2 = trim(nmlfile), &
!!$        & i = (/iostat_nml/) )
!!$    end if

    !-------------------------
    !  ϥǡθ̾μ
    !  Get individual information of output data
    rewind( unit_nml )
    iostat_nml = 0
    pos_nml = ''
    do while ( trim(pos_nml) /= 'APPEND' .and. iostat_nml == 0 )

      name                   = ''
      file                   = ''
      call HstNmlInfoInquire( &
        & gthstnml = gthstnml, &             ! (in)
        & interval_value = interval_value, & ! (out)
        & interval_unit = interval_unit, &   ! (out)
        & precision = precision, &           ! (out)
        & average = average, &               ! (out)
        & fileprefix = fileprefix )          ! (out)

      read( unit = unit_nml, &                      ! (in)
        &    nml = dyn_spectral_as83_history_nml, & ! (out)
        & iostat = iostat_nml )                     ! (out)
      inquire( unit = unit_nml, & ! (in)
        &  position = pos_nml )   ! (out)

      if ( iostat_nml == 0 ) then
        call MessageNotify( 'M', subname, &
          & 'NAMELIST group "%c" is loaded from "%c".', &
          & c1='dcmodel_sample_code_history_nml', c2=trim(nmlfile) )
        write(STDOUT, nml = dyn_spectral_as83_history_nml)

        call HstNmlInfoAdd( &
          & gthstnml = gthstnml, &             ! (in)
          & name = name, &                     ! (in)
          & file = file, &                     ! (in)
          & interval_value = interval_value, & ! (in)
          & interval_unit = interval_unit, &   ! (in)
          & precision = precision, &           ! (in)
          & average = average, &               ! (in)
          & fileprefix = fileprefix )          ! (in)

      else
        call MessageNotify( 'W', subname, &
          & 'NAMELIST group "%c" is not found in "%c" any more (iostat=%d).', &
          & c1='dcmodel_sample_code_history_nml', c2=trim(nmlfile), &
          & i = (/iostat_nml/) )
      end if
    end do

    close( unit_nml )

    !-----------------------------------------------------------------
    !  NAMELIST ѿʸ
    !  Substitute NAMELIST group to character arguments
    !-----------------------------------------------------------------
!!$    key00_       = key00

    !-----------------------------------------------------------------
    !  λ, 㳰
    !  Termination and Exception handling
    !-----------------------------------------------------------------
999 continue
    call StoreError( stat, subname, err, cause_c )
    call EndSub( subname )
  end subroutine DynSpAsNmlRead


end module dyn_spectral_as83
